/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.attributes.AttributeGroupsRetriever");

dojo.require("com.ibm.rdm.web.attributes.AttributeGroupStyleParser");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.util.domutils");


var AttributeGroupStyleParser = com.ibm.rdm.web.attributes.AttributeGroupStyleParser;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var Headers = com.ibm.rdm.web.server.Headers;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var domutils = com.ibm.rdm.web.util.domutils;


dojo.declare(
	"com.ibm.rdm.web.attributes.AttributeGroupsRetriever",
	null,
{
	// summary:
	//    This class is responsible for maintaining up to date information about
	//    attribute group style information for each project the user visits.
	// description:
	//    When asked to obain attribute group information for a particular
	//    project, this claass uses multi-fetch to get all attribute group style
	//    documents associated with a project and then compares the etags
	//    associated with these documents to see if there's already a cached
	//    AttributeGroupStyleParser associated with the document's etag.  If
	//    there IS a cached parser for the document at this revision level,
	//    it makes use of the cached AttributeGroupStyleParser.  Otherwise,
	//    it must be the case that we're either handling the FIRST visit to the
	//    associated project (in which case we need to do GETs for all attribute
	//    group style documents associated with this project) OR we're handling
	//    the case of an attribute group style document that's been updated
	//    since the last time we retrieved it (in which case we need to do a
	//    fresh GET for this attribute group style document).

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_projectURI: null,
	_scopedAttributeGroupNamespaces: null,
	_onAttributeGroupComplete: null,
	_callbackScope: null,
	_numAttributeGroupsToBeParsed: null,
	_numAttributeGroupsParsed: null,
	// _mapResourceURIToETag: an object that serves as a hashmap
	//    Each property represents a resource URI for an attribute group
	//    style document, and each property value represents the ETag value
	//    associated with this document when we last did a HTTP GET for it.
	_mapResourceURIToETag: null,
	// _mapETagToParser: an object that serves as a hashmap
	//    Each property represents an etag, and each property value represents
	//    an AttributeGroupStyleParser object that parsed the content for a
	//    particular attribute group style at a particular revision number
	//    (corresponding to the etag).
	_mapETagToParser: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	constructor: function(keywordArgs) {
		// keywordArgs.projectURI:
		//    The project URI (e.g. "https://localhost:9443/jazz/process/project-areas/_UKPzQKIyEd6w4-5D-Rt0cg").
		this._projectURI = keywordArgs.projectURI;
		this._mapResourceURIToETag = new Object;
		this._mapETagToParser = new Object;
		this._mapResourceURIToUpdateContext = new Object;
		this._mapResourceURIToUpdateString = new Object;
		this._mapResourceURIToToLastETagUsed = new Object;
	},

	fetchAttributeGroups: function(scopedAttributeGroupNamespaces, onAttributeGroupComplete, callbackScope) {
		// scopedAttributeGroupNamespaces:
		//    An array of attribute group style URIs, representing a subset
		//    of the project's attribute group styles that you wish to receive
		//    information about (e.g. ["http://com.ibm.rdm.attribute.group.FEAT"]).
		//    If passed as null, you will be informed about ALL of the project's
		//    attribute group styles.
		// onAttributeGroupComplete:
		//    A function that will be called back whenever a particular
		//    attribute group has been parsed or has been verified to be
		//    up to date.
		// callbackScope:
		//    The scope in which to call the passed 'onAttributeGroupComplete'.
		this._scopedAttributeGroupNamespaces = scopedAttributeGroupNamespaces;
		this._onAttributeGroupComplete = onAttributeGroupComplete;
		this._callbackScope = callbackScope;
		this._update();
	},

	destroy: function() {
		this.cancelOutstandingFetches();
	},

	cancelOutstandingFetches: function() {
		if (this._deferredArray) {
			for (var i = 0; i < this._deferredArray.length; i++) {
				var deferred = this._deferredArray[i];
				if (deferred.fired == -1) {
					deferred.cancel();
				}
			}
		}
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_update: function() {
		this._numAttributeGroupsParsed = 0;
		var query = (
			"dc:isPartOf=<" + com.ibm.rdm.web.FRONTING_SERVICE_ATTRIBUTE_GROUPS + ">"
		);
//		var query = (
//			"dc:format=\"" + com.ibm.rdm.web.common.MimeTypes.ATTRIBUTE_GROUP + "\""
//		);
		var namespaces = "dc=http://purl.org/dc/terms/,jazz=http://jazz.net/xmlns/foundation/1.0/";
		queryInfo = {
			query: query,
			namespaces: namespaces,
			projectURI: this._projectURI,
			skipCache: false,
			attributeGroups: true
		};
		var self = this;
		// TODO Convert this over to use multi-get.  This would help mainly in
		// the case of the VERY FIRST TIME attribute groups are loaded for a
		// particular project (as a separate GET would go out for every
		// attribute group style document).  After this first load for a project
		// multi-get wouldn't help much as it's probably rare that these
		// attribute group style documents are updated (and we'd be taking
		// advantage of our cache here so that we'd normally ONLY do the multi-
		// fetch and NOT do any GETs after the attribute groups are loaded for
		// a particular project).
		var onMultiFetchComplete = function(resourceInfos) {
			self._numAttributeGroupsToBeParsed = resourceInfos.length;
			dojo.forEach(
				resourceInfos,
				function(resourceInfo) {
					var resourceURI = resourceInfo.resourceURI;
					var revision = resourceInfo.etag;
					var lastCachedEtag = self._mapResourceURIToETag[resourceURI];
					if (!lastCachedEtag || lastCachedEtag !== revision) {
						// We need to do a GET for this resource.
						self._retrieveAttributeGroupStyleDocument(
							resourceURI, revision);
					} else {
						// Our cached value is up to date.
						var parser = self._mapETagToParser[revision];
						self._invokeCallbackIfNeeded(parser);
					}
				}
			);
		};
		var deferred = ProjectCache.executeMultiFetch(queryInfo);
		deferred.addCallback(onMultiFetchComplete);
	},

	_retrieveAttributeGroupStyleDocument: function(resourceURI, revision) {
		var self = this;
		var onAttributeGroupLoaded = function(response) {
			var parser = new AttributeGroupStyleParser({
				doc: response.doc
			});
			self._mapResourceURIToETag[resourceURI] = revision;
			self._mapETagToParser[revision] = parser;
			self._invokeCallbackIfNeeded(parser);
			return response;
		};
		var onAttributeGroupError = function(error) {
			if (! (error.dojoType && error.dojoType == "cancel")) {
				ErrorHandler.getErrorSectionWidget().showErrorNumber(
					7201, [], error
				);
			}
		};
		var deferred = RDMClient.doGet({
			uri: resourceURI,
			revision: revision
		});
		deferred.addCallback(onAttributeGroupLoaded);
		deferred.addErrback(onAttributeGroupError);
	},

	_invokeCallbackIfNeeded: function(parser) {
		var namespace = parser.getNamespace();
		if (this._scopedAttributeGroupNamespaces == null
			|| dojo.indexOf(this._scopedAttributeGroupNamespaces, namespace) != -1) {
			// Note: We're _only_ invoking callback if the 'scopedAttributeGroupNamespaces'
			// parameter to 'fetchAttributeGroups' was was passed as null OR it was passed
			// as non-null and we're able to find this parser's namespace within it.
			var id = parser.getId();
			var fieldProperties = parser.getFieldProperties();
			var displayName = parser.getDisplayName();
			var scope = (this._callbackScope || dojo.global);
			this._numAttributeGroupsParsed++;
			this._onAttributeGroupComplete.call(
				scope, namespace, id, displayName, fieldProperties,
				this._numAttributeGroupsParsed, this._numAttributeGroupsToBeParsed
			);
		}
	}
});


})();
