/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2008.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.common.CompactItemSelectionContainer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("com.ibm.rdm.web.util.domutils");


dojo.declare(
	"com.ibm.rdm.web.common.CompactItemSelectionContainer",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//	  Implements a selection area in which ";" delimited items appear.
	//    The items highlight when moused over and appear as selected when
	//    clicked on.  Supports both single and multiple selection.
	//    Optionally, you may add selection/unselection callbacks to be
	//    called whenever selection changes.

	templateString: "<div class=\"rdm-compact-item-selection-container\"><div class=\"content\" dojoAttachPoint=\"_contentDiv\"></div></div>",

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	// onItemSelected: callback
	//    Callback to invoke (in the context of 'callbackScope') whenever an
	//    item is selected.
	onItemSelected: null,
	// onItemUnselected: callback
	//    Callback to invoke (in the context of 'callbackScope') whenever an
	//    item is unselected.
	onItemUnselected: null,
	// callbackScope: optional scope
	//    'onItemSelected' and 'onItemUnselected' are invoked in this scope,
	//    if specified.
	callbackScope: null,
	// isMultiSelectionEnabled: boolean
	//    More than one item may be selected if this is passed as true.
	isMultiSelectionEnabled: false,
	// doInsertItemsInSortedOrder: boolean
	//    If passed as true, newly added items will be added in their sorted
	//    order.  Otherwise, if not specified or passed as false, newly added
	//    items will be added to the end of the currently displayed items.
	doInsertItemsInSortedOrder: false,
	// doInsertItemsInSortedOrder: string
	//    Pass a CSS classname here if you'd like to customize the appearance
	//    of the selectable text items that will appear in this container
	//    (i.e. in the CompactItemSelectionContainer).
	textItemCssClassName: "text-item",
	
	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_numSelectedItems: 0,
	_lastSelectedItemSpan: null,
	_mapSpanToItemKey: null,

	_onItemSelected: null,
	_onItemUnselected: null,
	_callbackScope: null,
	_isMultiSelectionEnabled: false,
	_doInsertItemsInSortedOrder: false,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_contentDiv: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._numSelectedItems = 0;
		this._lastSelectedItemSpan = null;
		this._mapSpanToItemKey = new Object;
		this._onItemSelected = this.onItemSelected;
		this._onItemUnselected = this.onItemUnselected;
		this._callbackScope = this.callbackScope;
		this._isMultiSelectionEnabled = this.isMultiSelectionEnabled;
		this._doInsertItemsInSortedOrder = this.doInsertItemsInSortedOrder;
		this._textItemCssClassName = this.textItemCssClassName;
	},

	addTextItem: function(itemText, /* Optional */ itemKey) {
		// summary:
		//    Add a new item to the list of displayed items.  If 'itemKey' is
		//    passed this key is the value that will be passed when select and
		//    unselect events occur (see fields 'onItemSelected' and
		//    'onItemUnselected').
		var itemSpan = this._createSpanForTextItem(itemText);
		this._setItemSpanId(itemSpan, (itemKey || itemText));
		this._mapSpanToItemKey[itemSpan.id] = (itemKey || itemText);
		this._insertItem(itemSpan);
	},

	// TODO: Perhaps add another function here in which we pass icon as well as text?

	getSelectedItems: function() {
		// summary:
		//    Get the selected items that are currently being displayed.
		//    For example, if only the following item was added:
		//        addTextItem("Must", "Must_ID");
		//    and this item was selected, the returned value for:
		//        getSelectedItems()
		//    would be:
		//        ["Must"]
		var items = [];
		for (itemSpan = this._contentDiv.firstChild;
			 itemSpan != null;
			 itemSpan = itemSpan.nextSibling) {
			var isSelected = dojo.hasClass(itemSpan, "select-item");
			if (isSelected) {
				var itemText = this._getItemText(itemSpan);
				items.push(itemText);
			}
		}
		return items;
	},

	getSelectedKeys: function() {
		// summary:
		//    Get the keys for the selected items that are currently being
		//    displayed.
		//    For example, if only the following item was added:
		//        addTextItem("Must", "Must_ID");
		//    and this item was selected, the returned value for:
		//        getSelectedItems()
		//    would be:
		//        ["Must_ID"]
		var itemKeys = [];
		for (itemSpan = this._contentDiv.firstChild;
			 itemSpan != null;
			 itemSpan = itemSpan.nextSibling) {
			var isSelected = dojo.hasClass(itemSpan, "select-item");
			if (isSelected) {
				var itemKey = this._mapSpanToItemKey[itemSpan.id];
				itemKeys.push(itemKey);
			}
		}
		return itemKeys;
	},

	setSelectedItems: function(itemTextLabels) {
		// summary:
		//    Clear selection and set the selection to items associated with
		//    the passed 'itemTextLabels'.
		//    Note: This does not invoke unselect callbacks (assuming they were
		//    specified during construction).
		this._unselectAllItems();
		for (itemSpan = this._contentDiv.firstChild;
			 itemSpan != null;
			 itemSpan = itemSpan.nextSibling) {
			var itemText = this._getItemText(itemSpan);
			var shouldBeSelected = (dojo.indexOf(itemTextLabels, itemText) != -1);
			if (shouldBeSelected) {
				this._selectItem(itemSpan);
			}
		}
	},

	setSelectedKeys: function(itemKeys) {
		// summary:
		//    Clear selection and set the selection to items associated with
		//    the passed 'itemKeys'.
		//    Note: This does not invoke unselect callbacks (assuming they were
		//    specified during construction).
		this._unselectAllItems();
		for (itemSpan = this._contentDiv.firstChild;
			 itemSpan != null;
			 itemSpan = itemSpan.nextSibling) {
			var itemKey = this._mapSpanToItemKey[itemSpan.id];
			var shouldBeSelected = (dojo.indexOf(itemKeys, itemKey) != -1);
			if (shouldBeSelected) {
				this._selectItem(itemSpan);
			}
		}
	},

	isAtLeastOneItemSelected: function() {
		// summary:
		//    Return true iff at least one item is currently selected.
		return (this._numSelectedItems > 0);
	},

	clearSelection: function() {
		// summary:
		//    Change all currently selected items back the unselected state.
		//    Note: This does not invoke unselect callbacks (assuming they were
		//    specified during construction).
		this._unselectAllItems();
	},

	removeAllItems: function() {
		// summary:
		//    Clear out this container.
		//    Note: This does not invoke unselect callbacks (assuming they were
		//    specified during construction).
		com.ibm.rdm.web.util.domutils.removeChildNodes(this._contentDiv);
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_insertItem: function(itemSpan) {
		if (this._doInsertItemsInSortedOrder) {
			// Add the new item (including delimiter) at the appropriate
			// sorted position.
			var textToInsert = this._getItemText(itemSpan);
	        var child = this._contentDiv.firstChild;
	        for (; child != null; child = child.nextSibling) {
				if (child.id) {
					// Note: We're skipping over "; " delimeter span nodes (w/ that .id check).
					var text = this._getItemText(child);
					if (text > textToInsert) {
						break;
					}
				}
	        }
	        if (child) {
				var delimiterSpanNode = this._createDelimiterSpanNode();
				if (child === this._contentDiv.firstChild) {
					// Place this new item at the start.
					dojo.place(itemSpan, child, "before");
					dojo.place(delimiterSpanNode, itemSpan, "after");
				} else {
					// Place this new item before the delimeter for 'child'.
					dojo.place(delimiterSpanNode, child.previousSibling, "before");
					dojo.place(itemSpan, delimiterSpanNode, "after");
				}
	        } else {
				// Place this new item at the end.
				this._addDelimiterIfNeeded();
	            this._contentDiv.appendChild(itemSpan);
	        }
		} else {
			// Add the new item (preceded by a delimiter) add the end of
			// the displayed items.
			this._addDelimiterIfNeeded();
			this._contentDiv.appendChild(itemSpan);
		}
	},

	_addDelimiterIfNeeded: function() {
		var delimiterSpanNode = null;
		if (this._contentDiv.firstChild) {
			delimiterSpanNode = this._createDelimiterSpanNode();
			this._contentDiv.appendChild(delimiterSpanNode);
		}
		return delimiterSpanNode;
	},

	_createDelimiterSpanNode: function() {
		return com.ibm.rdm.web.util.domutils.createSpannedTextNode("text-item", "; ");
	},

	_createSpanForTextItem: function(itemText) {
		var spanNode = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
			this._textItemCssClassName, itemText);
		this.connect(spanNode, "onmouseover", this._handleItemMouseOver);
		this.connect(spanNode, "onmouseout", this._handleItemMouseOut);
		this.connect(spanNode, "onclick", this._handleItemMouseClick);
		return spanNode;
	},

	_handleItemMouseClick: function(event) {
		var itemSpan = (dojo.isIE ? event.srcElement : event.target);
		var alreadySelected = dojo.hasClass(itemSpan, "select-item");
		var self = this;
		var selectItem = function(itemSpan) {
			self._selectItem(itemSpan);
			if (self._onItemSelected) {
				self._invokeSelectCallback(itemSpan);
			}
		};
		var unselectItem = function(itemSpan) {
			self._unselectItem(itemSpan);
			if (self._onItemUnselected) {
				self._invokeUnselectCallback(itemSpan);
			}
		};
		if (this._isMultiSelectionEnabled) {
			if (alreadySelected) {
				unselectItem(itemSpan);
			} else {
				selectItem(itemSpan);
			}
		} else {
			// Single selection only ...
			if (alreadySelected) {
				unselectItem(itemSpan);
			} else {
				if (this._lastSelectedItemSpan) {
					unselectItem(this._lastSelectedItemSpan);
				}
				selectItem(itemSpan);
			}
		}
	},

	_selectItem: function(itemSpan) {
		dojo.addClass(itemSpan, "select-item");
		this._numSelectedItems++;
		this._lastSelectedItemSpan = itemSpan;
	},

	_unselectItem: function(itemSpan) {
		dojo.removeClass(itemSpan, "select-item");
		this._numSelectedItems--;
		this._lastSelectedItemSpan = null;
	},

	_unselectAllItems: function() {
		for (itemSpan = this._contentDiv.firstChild;
			 itemSpan != null;
			 itemSpan = itemSpan.nextSibling) {
			var isSelected = dojo.hasClass(itemSpan, "select-item");
			if (isSelected) {
				dojo.removeClass(itemSpan, "select-item");
			}
		}
		self._lastSelectedItemSpan = null;
		this._numSelectedItems = 0;
	},

	_handleItemMouseOver: function(event) {
		var itemSpan = (dojo.isIE ? event.srcElement : event.target);
		dojo.addClass(itemSpan, "highlight-item");
	},

	_handleItemMouseOut: function(event) {
		var itemSpan = (dojo.isIE ? event.srcElement : event.target);
		dojo.removeClass(itemSpan, "highlight-item");
	},

	_invokeSelectCallback: function(itemSpan) {
		var itemKey = this._mapSpanToItemKey[itemSpan.id];
		var scope = (this._callbackScope || dojo.global);
		this._onItemSelected.call(scope, itemKey);
	},

	_invokeUnselectCallback: function(itemSpan) {
		var itemKey = this._mapSpanToItemKey[itemSpan.id];
		var scope = (this._callbackScope || dojo.global);
		this._onItemUnselected.call(scope, itemKey);
	},

	_setItemSpanId: function(itemSpan, itemKey) {
		itemSpan.id = (this.id + "#" + itemKey);
	},

	_getItemText: function(itemSpan) {
		return itemSpan.firstChild.nodeValue;
	}
});
