/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
	/*
	 * This is the main error reporting widget.
	 * It is initialized only once in the ResourcesPage.js and then reused throughout the application.
	 * While it provides a public API, it must be called via the static methods of ErrorHandler.js.
	 * Developers must make sure that they clean up the error sections once the error is shown.
	 * 
	 * 
	 * Examples:
	 * 
	 * dojo.require("com.ibm.rdm.web.common.ErrorHandler");
	 * 
	 * com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showXhrError(response);
	 * ....
	 * 
	 * or
	 * response.localization = {message: "ErrorRetrievingResource", substitutes: ["Document TEST"]};
     * com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showXhrError(response);
     * ....
     * 
     * or
     * com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showError({"message":"Brief error headline","details":"where, what, when..."});
	 */
	dojo.provide("com.ibm.rdm.web.common.ErrorSectionWidget");
	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("jazz.ui.HoverPopup");
	dojo.require("dojo.i18n");
	dojo.requireLocalization("com.ibm.rdm.web.common", "ErrorSectionWidget");
	
	var HoverPopup = null;
		
	dojo.declare("com.ibm.rdm.web.common.ErrorSectionWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "common/templates/ErrorSectionWidget.html"),
	id: "com-ibm-rdm-web-common-ErrorSectionWidget",
	_messages: dojo.i18n.getLocalization("com.ibm.rdm.web.common", "ErrorSectionWidget"),
	_resourcesPageWidget: null,

	postCreate: function() {
		this.inherited("postCreate", arguments);
		this._hide();
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	/*
	 * Error is typcally an xhr response passed to a callback function.
	 * Developers may also need to add custom localization messages manually:
	 * response.localization = {message: "ErrorRetrievingResource", substitutes: ["Document TEST"]};
	 */
	showXhrError: function(response, ioArgs) {
		var message = "";
		var doc = response;
		if (response.doc)
			doc = response.doc;
		
		//if deferred was cancelled, skip...
		if (response.dojoType && response.dojoType == "cancel") {
            return;
        }
		
		try {
			if (ioArgs.xhr.status == 404) {
				message = this._messages.Error_404;
		    }
			else if (ioArgs.xhr.status == 403) {
				message = this._messages.Error_403;
		    }
			else if (ioArgs.xhr.status == 410) {
				message = this._messages.Error_410;
		    }
		}
		catch (e) {
			var message = dojo.string.substitute(this._messages["errorHttpStatus"], [e]);
			console.log(message);
		}
		
		var details = dojo.isIE? response.description: response.stack;
		var localization = response.localization; //ex. {localization: {message: "ErrorRetrievingResource", substitutes: ["Document TEST"]}}
		var message;
		
		if (!localization && !message) { //default message
			message = dojo.string.substitute(
                this._messages.ErrorXhrCallback,
                [response.message]
            );
		}
		
		if (ioArgs) {
			var errorText = "";
			try {
				if (ioArgs.xhr && ioArgs.xhr.responseText && !errorText) {
					var responseText = ioArgs.xhr.responseText;
					
					if (responseText.search("<body>") >= 0)
						errorText = responseText.substring(responseText.search("<body>")+6, responseText.search("</body>"));
					else {
						var errorHTML = document.createElement("DIV");
						errorHTML.innerHTML = responseText;
						errorText = dojo.isIE?errorHTML.innerText:errorHTML.textContent;
					}
				}
			}
			catch(e) {
				var message = dojo.string.substitute(this._messages["errorHttpResponse"], [e]);
				console.log(message);
			}
			details = "<b>URL: " + ioArgs.args.url + "</b><p>" + errorText + "<p>" + details;
		}
		
		this._reportError({
			"message": message,
			"details": details,
			"localization": (localization!=null)?localization:null
		});
	},
	
	/*
	 * A simple wrapper for the main function
	 */
	showErrorMessage: function(message, /*String?*/details) {
		this._reportError({
			"message": message,
			"details": details
		});
	},
	
	/*
	 * errorId: id matching a message in ErrorSectionWidget.js NLS strings file
	 * errorParms: array of parms to be used for substitution in the error message, can be an empty array,
	 *   null or unspecified if not substitution is necessary
	 * details: an object representing more detail about the error, will be converted using toString,
	 *   can be null or unspecified
	 */
	showErrorNumber: function(errorId, errorParms, details) {
		var errorMessage = null;
		var detailsMessage = null;
		errorMessage = this._messages[errorId.toString()];
        if (errorMessage == undefined) {
        	errorMessage = this._messages["7200"];
			errorMessage = dojo.string.substitute(errorMessage, [errorId.toString()]);
        }
        if (errorParms != undefined && errorParms.length > 0) {
        	try {
				errorMessage = dojo.string.substitute(errorMessage, errorParms);
        	} catch (e) {
        		// Note: no need to externalize this string
        		detailsMessage = "Error occurred substituting error parameters: " + errorParms.toString();
        		if (details != undefined && details != null) {
        			detailsMessage = detailsMessage + "\n" + details.toString();
        		} 
        	}
        }
        if (detailsMessage == null) {
        	if (details == undefined || details == null) {
        		detailsMessage = "";
        	} else {
        		if (details.stack) {
        			// show the stack if this is a response error
        			detailsMessage = details.stack.toString();
        		} else {
        			detailsMessage = details.toString();
        		}
        	}
        }
		this._reportError({
			"message": errorMessage,
			"details": detailsMessage
		});
	},

	getErrorMessage: function(errorResponse) {
		var ioArgs = errorResponse.ioArgs;
		var message = "";
		try {
			if (ioArgs.xhr.status == 404) {
				message = this._messages.Error_404;
		    }
			else if (ioArgs.xhr.status == 403) {
				message = this._messages.Error_403;
		    }
			else if (ioArgs.xhr.status == 410) {
				message = this._messages.Error_410;
		    }
			else {
				message = dojo.string.substitute(
					this._messages["UnknownErrorMessage"], [ioArts.xhr.status]);
			}
		}
		catch (e) {
			var message = dojo.string.substitute(this._messages["errorHttpStatus"], [e]);
			console.log(message);
		}
		return message;
	},

	/*
	 * This should be called in the context of widget destroy() method
	 * com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().clearError();
	 */
	clearError: function() {
		this._hide();
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internal
	_test: function() {
		// bad error id
		this.showErrorNumber(
   			7210, [], "Bad error id"
		);
		// bad parms, too few
		this.showErrorNumber(
   			9999, ["abc"], "Too few parameters"
		);
		// bad parms, too few
		this.showErrorNumber(
   			9999, ["abc", "def"], "Too few parameters"
		);
		// bad parms, too few
		this.showErrorNumber(
   			9999, ["abc", "def"]
		);
		// bad parms, too many
		this.showErrorNumber(
   			9999, ["abc", "def", "ghi", "jkl"], "Too many parameters"
		);
		// test variations
		this.showErrorNumber(
   			7200, ["PARM"], "details 1"
		);
		this.showErrorNumber(
   			7200, null, "details 2"
		);
		this.showErrorNumber(
   			7200, [], null
		);
		this.showErrorNumber(
   			7200
		);
		// error conditions
		try {
			throw new NullPointerException();
		} catch (e) {
			this.showErrorNumber(9998, [1, 2, 3], e);
		};	
	},
		
	_reportError: function(error){
		var message;
		this._show();
		var localization = error.localization; //ex. {localization: {message: "ErrorRetrievingResource", substitutes: ["Document TEST"]}}
		if (localization != null) { //user specified message
			message = dojo.string.substitute(
                this._messages[localization.message],
                localization.substitutes
            );
		}
		else {
			message = error.message;
		}
		this._updateMessage(message);
		this._updateDetails(error.details);
	},
	
	_updateMessage: function(message) {	
		this._errorMessage.innerHTML = "";
		this._errorMessage.appendChild(document.createTextNode(message));
	},
	
	_updateDetails: function(details) {
		var msg = details;
		if (details == undefined || details == "" ) {
			msg = this._messages["7210"];
		}
		this._destroyPopup();
		this.HoverPopup = new jazz.ui.HoverPopup({
			around: [this._errorDetails],
			html: msg,
			openDelay: 300,
			orientation: "R"
		});
	},
	
	_hide: function() {
		this.domNode.style.visibility = "hidden";
		this.domNode.style.position = "absolute";
		this._resourcesPageWidget.updateResize();
	}, 
	
	_show: function() {
		this.domNode.style.visibility = "visible";
		this.domNode.style.position = "static";
		this._resourcesPageWidget.updateResize();
	},
	
	_destroyPopup: function() {
		if (this.HoverPopup)
			this.HoverPopup.destroyRecursive();
		this.HoverPopup = null;
	}
	
});
	
})();
