/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.rdm.web.common.UpdateContext");

com.ibm.rdm.web.common.UpdateContext = {
	// summary:
	//    The UpdateContext is used to facilitate avoiding needless HEAD
	//    requests in the context of opening a new page or refreshing the
	//    current page.
	// description:
	//    This class is meant to be used by caches that might be called more
	//    than one time in the context of opening or updating the page.  For
	//    example, if there were a UsersCache that had a getUsers function,
	//    two different controls on the same page may need to obtain the list
	//    of users.  If this cache's getUsers function were to call:
	//        com.ibm.rdm.web.common.UpdateContext.getValue()
	//    on the the first request and store this value away, it could then
	//    consult it on the subsequent getUsers call to test it against
	//    another call to:
	//        com.ibm.rdm.web.common.UpdateContext.getValue()
	//    to see that the there is no need to contact the server again,
	//    as the values returned by the first call are already known to be
	//    valid in this "update context" (of the page opening/refreshing
	//    operation).

	_updateContext: 0,
	
	// some classes may try to change update context at the same time
	// so we have to set an interval so that it would not happen
	// since it will defeat the purpose of caching
	UPDATE_INTERVAL: 5, //sec
	_lastUpdated:	null,

	getValue: function() {
		// summary:
		//    Gets the current update context.
		// description:
		//    Caches should make use of the returned value so that they can
		//    compare against it on subsequent calls to determine if these
		//    calls are being made within the same "update context" (i.e. page
		//    opening or refreshing).  The subsequent call would need to do
		//    something like this:
		//    |   lastUpdateContext = this._lastUpdateContext;
		//    |   currentUpdateContext = com.ibm.rdm.web.common.UpdateContext.getValue();
		//    |   if (lastUpdateContext === currentUpdateContext) {
		//    |       // Use last cached info (and don't make HEAD
		//    |       // requests).
		//    |   } else {
		//    |       // Make HEAD requests to make sure the cached info
		//    |       // is up to date.
		//    |   }
		//    |   this._lastUpdateContext = currentUpdateContext;
		return this._updateContext;
	},

	refreshHasOccurred: function() {
		// summary:
		//    Called whenever global refresh button is pressed.
		// description:
		//    The only class that should call this is the ResourcesPage.
		//    It's called to reset the update context whenever a refresh
		//    has been requested.
		if (this.isChangeContextAllowed()) {
			this._updateContext++;
		}
	},

	redirectHasOccurred: function() {
		// summary:
		//    Called whenever a new value has been set for window.location.href.
		// description:
		//    The only class that should call this is the ResourcesPage.
		//    Called whenever the user follows a link in our UI (e.g. to another
		//    resource, or to another view such as from ResourceViewer to
		//    ProjectDashboard).
		if (this.isChangeContextAllowed()) {
			this._updateContext++;
		}
	},

	cachedArtifactHasBeenSaved: function() {
		// summary:
		//    Called whenever an artifact that might be found in a cache has
		//    been successfully saved.
		// description:
		//    If an artifact has just been saved client side AND is an artifact
		//    that is normally asked for from a cache, we must make sure to
		//    update the UpdateContext, as we want subsequent gets in the
		//    cache to know that the server needs to be contacted (even though
		//    a page redirect or refresh has not occurred).
		//    For example, when a vote is cast in a Review, this function is
		//    called so that other viewers that need to know of the changes
		//    can make use of a ReviewCache to get the newly saved artifact
		//    information.
		this._updateContext++;
	},
	
	isChangeContextAllowed: function() {
		var now = new Date().getTime();
		if (this._lastUpdated && this.UPDATE_INTERVAL*1000 < (now - this._lastUpdated)) {
			this._lastUpdated = now;
			return true;
		}
		else if(this._lastUpdated) {
			return false;
		}
		this._lastUpdated = now;
		return true;
	}
};

})();
