/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.project.AttributeValueSelector");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.ValidationTextBox");
dojo.require("dijit.form.NumberTextBox");
dojo.require("dijit.form.TextBox");
dojo.require("com.ibm.rdm.web.attributes.AttributeGroupsCache");
dojo.require("com.ibm.rdm.web.common.CompactItemSelectionContainer");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.requireLocalization("com.ibm.rdm.web.project", "AttributeValueSelector");


dojo.declare(
	"com.ibm.rdm.web.project.AttributeValueSelector",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//	  Provides a widget that enables users to specify a single
	//    attribute value from amongst the various attribute groups found
	//    within a particular project.
	// description:
	//    Each project contains a set of attribute groups.  This widget
	//    enables users to select a particular attribute value from
	//    one of the attribute groups associated with the project.
	//    For example, the user might specify that he/she wants
	//    "Feature [attribute group] whose Priority [enum attribute] is
	//    'Must' or 'Should'".  Another example might be "Market
	//    [attribute group] whose Contact [integer attribute] is 100".

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/AttributeValueSelector.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	projectURI: null,
	onContentAreaHeightHasChanged: null,
	// onInvokeApplyAction: Callback
	//    This callback is invoked whenever an implicit apply is performed.
	//    For example, if the user sets the Supplemental -> Contact text value
	//    to "Palin" and then hits the Enter key, we'll invoke this callback
	//    to apply the appropriate action.
	onInvokeApplyAction: null,
	// onEnableApplySection: Callback
	//    This callback is invoked whenever it's noticed that the user has
	//    provided enough information to enable an Apply action.  For example,
	//    as soon as the user has selected one of the Feature -> Priority values
	//    (e.g. "High") we'll invoke this callback.
	onEnableApplySection: null,
	// onDisableApplySection: Callback
	//    This callback is invoked whenever it's noticed that the user has
	//    hasn't provided enough information to enable an Apply action.  For
	//    example, if the user removes the only selected Feature -> Priority
	//    (e.g. "High") we'll invoke this callback.
	onDisableApplySection: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	// _attributeGroupSelectionContainer
	//    Container for enabling selecting a particular attribute group
	//    (e.g. "Feature").
	_attributeGroupSelectionContainer: null,
	// _attributeSelectionContainer
	//    Container for enabling selecting a particular field within an
	//    attribute group (e.g. the "Type" field of "Feature").
	_attributeSelectionContainer: null,
	// _attributeValueWidget
	//    Holds the widget used for selecting an appropiate value.
	//    If the type is "Enum" or "Boolean" this will be set to a
	//    CompactItemSelectionContainer for enabling selecting one or more enum
	//    field values (e.g. the "Functional" and/or "Non-Functional" enum
	//    values within "Feature's "Type" field) or boolean field values
	//    ("true" or "false").
	//    Otherwise, the type is either "Date", "Decimal", "Float", "Integer"
	//    or "URI", and for these types we use and instance of a dojo
	//    dijit.form.TextBox (or a class derived from that class).
	_attributeValueWidget: null,
	// _mapAttributeGroupIdToFieldProperties: an object that serves as a hashmap
	//    The keys are attribute group ids (e.g. "FEAT") and the values are the
	//    associated field properties (as described in the
	//    AttributeGroupStyleParser.getFieldProperties function).
	_mapAttributeGroupIdToFieldProperties: null,
	_mapAttributeGroupIdToDisplayName: null,
	_mapAttributeGroupIdToNamespace: null,
	_selectedAttributeGroupId: null,
	_selectedAttributeId: null,
	_onRefreshComplete: null,
	_refreshInfo: {
		selectedAttributeGroupId: null,
		selectedAttributeId: null,
		selectedEnumOrBooleanFieldIds: null
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_attributeGroupSelectorDiv: null,
	_attributeSectionDiv: null,
	_attributeSelectorDiv: null,
	_attributeValueSectionDiv: null,
	_attributeValueSelectorDiv: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._attributeGroupSelectionContainer =
			new com.ibm.rdm.web.common.CompactItemSelectionContainer({
				onItemSelected: this._onAttributeGroupSelected,
				onItemUnselected: this._onAttributeGroupUnselected,
				callbackScope: this,
				isMultiSelectionEnabled: false,
				doInsertItemsInSortedOrder: true
		});
		this._attributeGroupSelectorDiv.appendChild(this._attributeGroupSelectionContainer.domNode);
		this._attributeSectionDiv.style.display = "none";
		this._selectedAttributeGroupId = null;
		this._selectedAttributeId = null;
		this._refresh();
	},

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "AttributeValueSelector");
	},

	destroy: function() {
		this.inherited(arguments);
		if (this._attributeGroupSelectionContainer) {
			this._attributeGroupSelectionContainer.destroyRecursive();
		}
		if (this._attributeSelectionContainer) {
			this._attributeSelectionContainer.destroyRecursive();
		}
		if (this._attributeValueWidget) {
			this._attributeValueWidget.destroyRecursive();
		}
	},

	refresh: function(onRefreshComplete) {
		this._onRefreshComplete = onRefreshComplete;
		var attributeGroupId = this._selectedAttributeGroupId;
		var attributeId = this._selectedAttributeId;
		var enumOrBooleanFieldIds = null;
		if (attributeId) {
			var fieldProperties = this._mapAttributeGroupIdToFieldProperties[attributeGroupId];
			if (fieldProperties
				&& (fieldProperties[attributeId].type === "Enum"
					|| fieldProperties[attributeId].type === "Boolean")
				) {
				enumOrBooleanFieldIds = this._attributeValueWidget.getSelectedKeys();
			}
		}
		this._refreshInfo = {
			selectedAttributeGroupId: attributeGroupId,
			selectedAttributeId: attributeId,
			selectedEnumOrBooleanFieldIds: enumOrBooleanFieldIds
		};
		this.onDisableApplySection();
		if (attributeGroupId) {
			this._onAttributeGroupUnselected(attributeGroupId);
		}
		this._attributeGroupSelectionContainer.removeAllItems();
		if (this._attributeSelectionContainer) {
			this._attributeSelectionContainer.removeAllItems();
		}
		if (this._attributeValueWidget instanceof com.ibm.rdm.web.common.CompactItemSelectionContainer) {
			this._attributeValueWidget.removeAllItems();
		}
		this._refresh();
	},

	clearSelection: function() {
		this._unselectAllItems();
	},

	focus: function() {
		this._focus();
	},

	isAttributeValueSpecified: function() {
		return this._isAttributeValueSpecified();
	},

	getAttributeValuesInfo: function() {
		// summary:
		//    Returns a property list in the following format:
		//        {
		//            attributeType: < Boolean | Date | Decimal | Enum | Float | Integer | String | URI >,
		//            attributeGroupNamespace: <attributeGroupNamespace>,
		//            attributeGroupId: <attributeGroup>,
		//            attributeNameId: <attributeGroup>,
		//            attributeValues: [ <attributeValue> * ]
		//        }
		//    Examples:
		//      An enum ...
		//        {
		//            attributeType: "Enum",
		//            attributeGroupNamespace: "http://com.ibm.rdm.attribute.group.ORIGIN",
		//            attributeGroupId: "ORIGIN",
		//            attributeNameId: "Origin_ID",
		//            attributeValues: [ "Quality_ID", "Development_ID" ]
		//        }
		//      A text field ...
		//        {
		//            attributeType: "String",
		//            attributeGroupNamespace: "http://com.ibm.rdm.attribute.group.ORIGIN",
		//            attributeGroupId: "ORIGIN",
		//            attributeNameId: "Name_ID",
		//            attributeValues: [ "freddy" ]
		//        }
		// 
		var attributeValues = this._getQueryAttributeValues();
		var fieldProperties = this._mapAttributeGroupIdToFieldProperties[this._selectedAttributeGroupId];
		var attributeDisplayName = fieldProperties[this._selectedAttributeId].displayName;
		var attributeType = fieldProperties[this._selectedAttributeId].type;
		if (attributeType == "URI") {
			if (attributeValues && attributeValues[0] && attributeValues[0].indexOf('http')!= 0) {
				attributeValues[0] = 'http://' + attributeValues[0];
			}
		}
		return {
			attributeType: attributeType,
			attributeGroupNamespace:
				this._mapAttributeGroupIdToNamespace[this._selectedAttributeGroupId],
			attributeGroupId: this._selectedAttributeGroupId,
			attributeNameId: this._selectedAttributeId,
			attributeValues: attributeValues
		};
	},

	isAttributeValuesInfoStillValid: function(attributeValuesInfo) {
		var isAttributeValuesInfoStillValid = true;
		var attributeGroupId = attributeValuesInfo.attributeGroupId;
		var attributeNameId = attributeValuesInfo.attributeNameId;
		var attributeType = attributeValuesInfo.attributeType;
		var attributeValues = attributeValuesInfo.attributeValues;
		var fieldProperties = this._mapAttributeGroupIdToFieldProperties[attributeGroupId];
		if (fieldProperties) {
			var fieldProperty = fieldProperties[attributeNameId];
			if (fieldProperty && fieldProperty.type === attributeType) {
				if (attributeType === "Enum") {
					var mapEnumFieldValueIdToFieldValue = fieldProperty.enumValues;
					for (var i = 0; i < attributeValues.length; i++) {
						enumFieldValueId = attributeValues[i];
						if (! mapEnumFieldValueIdToFieldValue[enumFieldValueId]) {
							isAttributeValuesInfoStillValid = false;
							break;
						}
					}
				}
			} else {
				isAttributeValuesInfoStillValid = false;
			}
		} else {
			isAttributeValuesInfoStillValid = false;
		}
		return isAttributeValuesInfoStillValid;
	},

	getSummaryLabel: function() {
		var fieldProperties = this._mapAttributeGroupIdToFieldProperties[this._selectedAttributeGroupId];
		var attributeDisplayName = fieldProperties[this._selectedAttributeId].displayName;
		var attributeValues = this._getSummaryAttributeValues();
		var label = attributeDisplayName + " = ";
		for (var i = 0; i < attributeValues.length; i++) {
			var value = attributeValues[i];
			if (i < (attributeValues.length-1)) {
				label += (value + ", ");
			} else {
				label += value;
			}
		}
		return label;
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_unselectAllItems: function() {
		this._attributeGroupSelectionContainer.clearSelection();
		if (this._selectedAttributeGroupId) {
			this._onAttributeGroupUnselected(this._selectedAttributeGroupId);
			this._selectedAttributeGroupId = null;
			this._selectedAttributeId = null;
			this._removeAttributeValueWidgetIfNeeded();
		}
	},

	_refresh: function() {
		this._mapAttributeGroupIdToFieldProperties = new Object;
		this._mapAttributeGroupIdToDisplayName = new Object;
		this._mapAttributeGroupIdToNamespace = new Object;
		com.ibm.rdm.web.attributes.AttributeGroupsCache.fetchAttributeGroups(
			this.projectURI,
			null,			   // ALL attribute groups (i.e. not scoped).
			this._onSingleAttributeGroupRetrieved,
			this
		);
	},

	_onSingleAttributeGroupRetrieved: function(namespace, id, displayName, fieldProperties, attrGroupIndex, totalAttrGroups) {
		this._mapAttributeGroupIdToFieldProperties[id] = fieldProperties;
		this._mapAttributeGroupIdToDisplayName[id] = displayName;
		this._mapAttributeGroupIdToNamespace[id] = namespace;
		this._attributeGroupSelectionContainer.addTextItem(displayName, id);
		if (attrGroupIndex === totalAttrGroups) {
			this._resetSelectionToValuesInPlaceBeforeRefresh();
			if (this._onRefreshComplete) {
				this._onRefreshComplete();
			}
		}
	},

	_resetSelectionToValuesInPlaceBeforeRefresh: function() {
		var selectedAttributeGroupId = this._refreshInfo.selectedAttributeGroupId;
		var selectedAttributeId = this._refreshInfo.selectedAttributeId;
		var selectedEnumOrBooleanFieldIds = this._refreshInfo.selectedEnumOrBooleanFieldIds;
		var self = this;
		var attributeGroupStillExists = function() {
			return self._mapAttributeGroupIdToFieldProperties[selectedAttributeGroupId] != null;
		};
		var attributeFieldStillExists = function() {
			var fieldProperties = self._mapAttributeGroupIdToFieldProperties[selectedAttributeGroupId];
			return fieldProperties[selectedAttributeId];
		};
		// Reselect the attribute group that was previously selected
		// (i.e. before a Refresh was performed).
		if (selectedAttributeGroupId && attributeGroupStillExists()) {
			this._attributeGroupSelectionContainer.setSelectedKeys([selectedAttributeGroupId]);
			this._onAttributeGroupSelected(selectedAttributeGroupId);
			// Reselect the attribute field that was previously selected
			// (i.e. before a Refresh was performed).
			if (selectedAttributeId && attributeFieldStillExists()) {
				// This attribute group field still exists, so select it if appropriate.
				if (this._attributeSelectionContainer) {
					this._attributeSelectionContainer.setSelectedKeys([selectedAttributeId]);
					if (this._attributeValueWidget instanceof com.ibm.rdm.web.common.CompactItemSelectionContainer) {
						this._populateValueSelectionContainer(selectedAttributeId, this._attributeValueWidget);
					}
					this._onAttributeSelected(selectedAttributeId, true);
				}
				// If the type of this field is "Enum" or "Boolean" we need to
				// make sure that the corresponding selection container shows
				// with the same selected values [for any that still exist]
				// that were selected before the user performed the Refresh.
				if (selectedEnumOrBooleanFieldIds) {
					if (this._attributeValueWidget
						&& this._attributeValueWidget instanceof com.ibm.rdm.web.common.CompactItemSelectionContainer
						) {
						this._attributeValueWidget.setSelectedKeys(selectedEnumOrBooleanFieldIds);
						this._handleEnumOrBooleanValueSelectionHasChanged();
					}
				}
			}
		}
	},

	_onAttributeGroupSelected: function(attributeGroupId) {
		this.onDisableApplySection();
		this._selectedAttributeGroupId = attributeGroupId;
		var self = this;
		var addAttributeGroupItemsToContainer = function() {
			var fieldProperties =
				self._mapAttributeGroupIdToFieldProperties[attributeGroupId];
			for (var i = 0; i < fieldProperties.length; i++) {
				var fieldProperty = fieldProperties[i];
				self._attributeSelectionContainer.addTextItem(
					fieldProperty.displayName, fieldProperty.id);
			}
		}
		if (this._attributeSelectionContainer) {
			// Reset the content of the previously created container.
			this._attributeSelectionContainer.removeAllItems();
			addAttributeGroupItemsToContainer();
		} else {
			// Create a container to hold the attributes.
			var attributeSelectionContainer =
				new com.ibm.rdm.web.common.CompactItemSelectionContainer({
					onItemSelected: this._onAttributeSelected,
					onItemUnselected: this._onAttributeUnselected,
					callbackScope: this,
					isMultiSelectionEnabled: false
			});
			this.onDisableApplySection();
			this._attributeSelectorDiv.appendChild(
				attributeSelectionContainer.domNode);
			this._attributeSelectionContainer = attributeSelectionContainer;
			addAttributeGroupItemsToContainer();
		}
		this._attributeSectionDiv.style.display = "";
		this.onContentAreaHeightHasChanged();
	},

	_onAttributeGroupUnselected: function(attributeGroupId) {
		this._selectedAttributeGroupId = null;
		this._attributeSectionDiv.style.display = "none";
		this._attributeValueSectionDiv.style.display = "none";
		this.onContentAreaHeightHasChanged();
	},

	_onAttributeSelected: function(attributeId, doPreserveExistingAttributeValueWidget) {
		this._selectedAttributeId = attributeId;
		if (doPreserveExistingAttributeValueWidget) {
			if (this._attributeValueWidget instanceof com.ibm.rdm.web.common.CompactItemSelectionContainer) {
				this._handleEnumOrBooleanValueSelectionHasChanged();
			} else if (this._attributeValueWidget instanceof dijit.form.DateTextBox) {
				this._onDateValueChanged();
			} else if (this._attributeValueWidget instanceof dijit.form.NumberTextBox) {
				this._onNumberTextBoxInputChanged();
			} else if (this._attributeValueWidget instanceof dijit.form.TextBox) {
				this._onTextBoxInputChanged();
			}
		} else {
			this._removeAttributeValueWidgetIfNeeded();
			this._addAttributeValueWidget(attributeId);
		}
		this._attributeValueSectionDiv.style.display = "";
		this.onContentAreaHeightHasChanged();
	},

	_onAttributeUnselected: function(attributeId) {
		this._selectedAttributeId = null;
		this._attributeValueSectionDiv.style.display = "none";
		this.onContentAreaHeightHasChanged();
	},

	_onEnumOrBooleanValueSelected: function(attributeId) {
		// Note: This callback is ONLY invoked when the '_attributeValueWidget'
		// is a CompactItemSelectionContainer.
		this._handleEnumOrBooleanValueSelectionHasChanged();
	},

	_onEnumOrBooleanValueUnselected: function(attributeId) {
		// Note: This callback is ONLY invoked when the '_attributeValueWidget'
		// is a CompactItemSelectionContainer.
		this._handleEnumOrBooleanValueSelectionHasChanged();
	},

	_handleEnumOrBooleanValueSelectionHasChanged: function() {
		if (this._attributeValueWidget.isAtLeastOneItemSelected()) {
			this.onEnableApplySection();
		} else {
			this.onDisableApplySection();
		}
	},

	_onDateValueChanged: function() {
		// Note: This function is ONLY invoked when the '_attributeValueWidget'
		// is some form of TextBox.
		var value = this._attributeValueWidget.attr('value');
		if (value != "") {
			this.onEnableApplySection();
		} else {
			this.onDisableApplySection();
		}
	},

	_onTextBoxInputChanged: function() {
		var textBox = this._attributeValueWidget;
		var value = textBox.attr('value');
		if (value === ""
			|| ((textBox instanceof dijit.form.ValidationTextBox)
				&& (! textBox.isValid(true)))) {
			this.onDisableApplySection();
		} else {
			this.onEnableApplySection();
		}
	},

	_onNumberTextBoxInputChanged: function() {
		var textBox = this._attributeValueWidget;
		var value = textBox.attr('value');
		if (isNaN(value)
			|| ((textBox instanceof dijit.form.ValidationTextBox)
				&& (! textBox.isValid(true)))) {
			this.onDisableApplySection();
		} else {
			this.onEnableApplySection();
		}
	},

	_addAttributeValueWidget: function(attributeId) {
		var fieldProperties =
			this._mapAttributeGroupIdToFieldProperties[this._selectedAttributeGroupId];
		var fieldProperty = fieldProperties[attributeId];
		var attributeValueWidget = null;
		var self = this;
		// Callbacks for TextBox
		var onTextBoxKeyPress = function(event) {
			var value = self._attributeValueWidget.attr('value');
			if (value && dojo.trim(value) != "" && event.keyCode == 13) { // 13 is Enter key
				self.onInvokeApplyAction();
			}
		};
		// Callbacks for NumberTextBox
		var onNumberTextBoxKeyPress = function(event) {
			var value = self._attributeValueWidget.attr('value');
			if (!isNaN(value) && event.keyCode == 13) { // 13 is Enter key
				self.onInvokeApplyAction();
			}
		};
		// Set attributeValueWidget to an appropriate widget.
		switch (fieldProperty.type) {
			case "URI":
			case "String":
				// TODO Add some kind of validator if it's a "URI"?
				attributeValueWidget = new dijit.form.TextBox({
					intermediateChanges: true,
					_onKeyPress: function(event) {
						onTextBoxKeyPress(event);
						this.inherited("_onKeyPress", arguments);
					},
					onChange: function(newValue) {
						this.inherited("onChange", arguments);
						self._onTextBoxInputChanged();
					}
				});
				break;
			case "Date":
				// TODO Move these DataTextBox width specifications to css?
				invalidMessage: self._getInvalidMessage(fieldProperty.type),
				attributeValueWidget = new dijit.form.DateTextBox({
					style: "width: 100px;",
					onChange: function() {
						self._onDateValueChanged();
		   			}
				});
				break;
			case "Decimal":
			case "Integer":
				attributeValueWidget = new dijit.form.NumberTextBox({
					intermediateChanges: true,
					invalidMessage: self._getInvalidMessage(fieldProperty.type),
					_onKeyPress: function(event) {
						onNumberTextBoxKeyPress(event);
						this.inherited("_onKeyPress", arguments);
					},
					onChange: function(event) {
						this.inherited("onChange", arguments);
						self._onNumberTextBoxInputChanged();
					}
				});
				break;
			case "Float":
				attributeValueWidget = new dijit.form.NumberTextBox({
					intermediateChanges: true,
					invalidMessage: self._getInvalidMessage(fieldProperty.type),
					_onKeyPress: function(event) {
						onNumberTextBoxKeyPress(event);
						this.inherited("_onKeyPress", arguments);
					},
					onChange: function(event) {
						this.inherited("onChange", arguments);
						self._onNumberTextBoxInputChanged();
					}
				});
				break;
			case "Boolean":
				attributeValueWidget = new com.ibm.rdm.web.common.CompactItemSelectionContainer({
					onItemSelected: this._onEnumOrBooleanValueSelected,
					onItemUnselected: this._onEnumOrBooleanValueUnselected,
					callbackScope: this,
					isMultiSelectionEnabled: false
				});
				this._populateValueSelectionContainer(attributeId, attributeValueWidget);
				break;
			case "Enum":
				attributeValueWidget = new com.ibm.rdm.web.common.CompactItemSelectionContainer({
					onItemSelected: this._onEnumOrBooleanValueSelected,
					onItemUnselected: this._onEnumOrBooleanValueUnselected,
					callbackScope: this,
					isMultiSelectionEnabled: true
				});
				this._populateValueSelectionContainer(attributeId, attributeValueWidget);
				break;
		}
		this.onDisableApplySection();
		this._attributeValueSelectorDiv.appendChild(attributeValueWidget.domNode);
		this._attributeValueWidget = attributeValueWidget;
	},

	_populateValueSelectionContainer: function(attributeId, attributeValueWidget) {
		// Precondition: the selected attribute field is either of type 
		// "Boolean" or "Enum".
		var fieldProperties =
			this._mapAttributeGroupIdToFieldProperties[this._selectedAttributeGroupId];
		var fieldProperty = fieldProperties[attributeId];
		switch (fieldProperty.type) {
			case "Boolean":
				attributeValueWidget.addTextItem(this._messages["true"], "true");
				attributeValueWidget.addTextItem(this._messages["false"], "false");
				break;
			case "Enum":
				var enumValues = fieldProperty.enumValues;
				for (var i = 0; i < enumValues.length; i++) {
					var enumValue = enumValues[i];
					attributeValueWidget.addTextItem(enumValue.displayName, enumValue.id);
				}
				break;
		}
	},

	_removeAttributeValueWidgetIfNeeded: function() {
		if (this._attributeValueWidget) {
			this._attributeValueWidget.destroyRecursive();
			this._attributeValueWidget = null;
			com.ibm.rdm.web.util.domutils.removeChildNodes(this._attributeValueSelectorDiv);
		}
	},

	_getInvalidMessage: function(fieldType) {
		// summary:
		//    Get the message that should be displayed to the user if he/she
		//    enters invalid characters into the value input (i.e.
		//    into the this._attributeValueWidget).
		// fieldType:
		//    A value from a fieldProperty.type (see AttributeGroupStyleParser).
		var invalidMessage = "";
		switch (fieldType) {
			case "URI":
				invalidMessage = this._messages["invalidURI"];
				break;
			case "Date":
				invalidMessage = this._messages["invalidDate"];
				break;
			case "Decimal":
				invalidMessage = this._messages["invalidDecimal"];
				break;
			case "Integer":
				invalidMessage = this._messages["invalidInteger"];
				break;
			case "Float":
				invalidMessage = this._messages["invalidFloat"];
				break;
			case "Boolean":
			case "Enum":
			case "String":
				// Note: No validator is needed for types Boolean, Enum and
				// String.
				break;
		}
		return invalidMessage;
	},

	_isAttributeValueSpecified: function() {
		var isAttributeValueSpecified = false;
		if (this._attributeValueWidget) {
			var widget = this._attributeValueWidget;
			if (widget instanceof com.ibm.rdm.web.common.CompactItemSelectionContainer) {
				isAttributeValueSpecified = widget.isAtLeastOneItemSelected();
			}
			else if (widget instanceof dijit.form.NumberTextBox) {
				var value = widget.attr('value');
				isAttributeValueSpecified = (! isNaN(value));
			}
			else if (widget instanceof dijit.form.TextBox) {
				var value = widget.attr('value');
				isAttributeValueSpecified = (value != undefined && value.length > 0);
			}
		}
		return isAttributeValueSpecified;
	},

	_focus: function() {
		if (this._attributeValueWidget) {
			var widget = this._attributeValueWidget;
			if (widget
				&& (widget instanceof dijit.form.TextBox
					|| widget instanceof dijit.form.NumberTextBox)) {
				dijit.selectInputText(widget.textbox);
			}
		}
	},

	_getSummaryAttributeValues: function() {
		var attributeValues = [];
		if (this._attributeValueWidget) {
			var widget = this._attributeValueWidget;
			if (widget instanceof com.ibm.rdm.web.common.CompactItemSelectionContainer) {
				attributeValues = widget.getSelectedItems();
			}
			else if (widget instanceof dijit.form.DateTextBox) {
				var value = widget.attr('displayedValue');
				attributeValues = [value];
			}
			else if (widget instanceof dijit.form.TextBox) {
				var value = widget.attr('value');
				attributeValues = [value];
			}
		}
		return attributeValues;
	},

	_getQueryAttributeValues: function() {
		var attributeValues = [];
		if (this._attributeValueWidget) {
			var widget = this._attributeValueWidget;
			if (widget instanceof com.ibm.rdm.web.common.CompactItemSelectionContainer) {
				attributeValues = widget.getSelectedKeys();
			}
			else if (widget instanceof dijit.form.DateTextBox) {
				// E.g. if date displayed in calendar is 9/3/08 we want to return ["2008-09-03"]
				var twoDigitString = function(number) {
					return (
						number > 9 ?
						number.toString() :
						"0" + number.toString()
					);
				}
				var date = widget.getValue();
				var dayOfMonthString = twoDigitString(date.getDate());
				var monthOfYearString = twoDigitString(date.getMonth() + 1);
				attributeValues = [
					date.getFullYear().toString() + "-"
					+ monthOfYearString + "-"
					+ dayOfMonthString
				];
			}
			else if (widget instanceof dijit.form.TextBox) {
				var value = widget.attr('value');
				attributeValues = [value];
			}
		}
		return attributeValues;
	}
});
