/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.FoldersSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.parser");
dojo.require("com.ibm.rdm.web.project.DashboardSection");
dojo.require("com.ibm.rdm.web.project.projectutils");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.common.PublicTagsParser");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");

var JSON_PERMALINK_PROP = "permalink";
var JSON_NAME_PROP = "name";
var JSON_TYPE_PROP = "type";
var JSON_TYPE_VALUES = { stub:"stub", folder:"folder", loading:"loading" };
var JSON_PARENT_PROP = "parent";
var JSON_CHILDREN_PROP = "children";

var queryutils = com.ibm.rdm.web.util.queryutils;
var projectutils = com.ibm.rdm.web.project.projectutils;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;

/**
 * Presents a section view on the left in the project dashboard to enable users
 * to filter by folders.
 */
dojo.declare("com.ibm.rdm.web.project.FoldersSection", com.ibm.rdm.web.project.DashboardSection, {

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields

	// _SELECTION_INCLUDES_CHILD_FOLDERS: boolean
	//    Perhaps in the future we may wish to have some kind of user preference
	//    setting for whether selecting a folder should result in selecting all
	//    child folders as well.  It used to be that we would always
	//    automatically select child folders whenever a folder is selected.
	//    However, we've decided to change this default behavior to NOT do this
	//    automatically (instead, selecting a folder results in only selecting
	//    that single folder) ... so we're setting this to 'false' for now ...
	_SELECTION_INCLUDES_CHILD_FOLDERS: false,
	isFilterDeselectionByFolderAllowed:false,   //Determines whether clicking on an already selected folder removes the filter in the viewer.

    _tree: null,
    _folderStore: null,

    _selectionInfo: {
        treeNode: null,
        dataItem: null
    },
	_initialRootSelectionHasOccurred: false,
	_enableClearFiltersOnStart: true,

    _connectHandles: null,
    _expandNodeHandle: null,
                
    // This attribute will ONLY hold a value when a refresh operation is in
    // progress.  (Note: This attribute is null'd out in the constructor.)
    _refreshInfo: {
        // When a refresh is asked for, selectedFolderURI will
        // capture the selected folder so that it may be re-selected after a
        // brand new tree has been added to the DOM.
        selectedFolderURI: null,
		// When selectedFolderURI is set, this array will hold the URIs of
		// the parent folders for the selected folder (note: the length of
		// this array will be 0 if the selected folder represents a toplevel
		// folder).
        parentFolderURIs: null,
        // The callback to invoke when the refresh operation has completed (i.e.
        // after a new tree reflecting a fresh folder listing has been added
        // to the DOM).
        refreshCompletedCallback: null
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    constructor: function() {
        this._refreshInfo = null;
        this._selectionInfo = {
            treeNode: null,
            dataItem: null
        };
		this._initialRootSelectionHasOccurred = false;
    },

    postCreate: function() {
        this.inherited("postCreate", arguments);
    },

    startup: function() {
        this.inherited("startup", arguments);
    },

    destroy: function() {
        this.inherited("destroy", arguments);
        this._removeSelection(true);
        if (this._tree) {
            this._tree.destroyRecursive();
        }
        if (this._folderStore) {
            this._folderStore.destroy();
        }
    },

    getSelectedFolders: function() {
        // summary:
        //    Return an array of strings where each element of the array
        //    represents the resource URI for a selected folder.
        var selectedFolderResourceURIs = [];
		if (this._initialRootSelectionHasOccurred) {
			if (this._SELECTION_INCLUDES_CHILD_FOLDERS) {
				if (this._selectionInfo.treeNode) {
					// The selection includes the top-most selected folder as well
					// as all descendant folders.
					this._appendDescendantFolders(
	                	this._selectionInfo.dataItem, selectedFolderResourceURIs);
				}
			} else {
				if (this._selectionInfo.treeNode) {
					// The selection includes just the single selected folder.
					var permalink = this._folderStore.getValue(
						this._selectionInfo.dataItem, JSON_PERMALINK_PROP);
					selectedFolderResourceURIs = [permalink];
				}
			}
		} else {
			// Handle the case of !this._initialRootSelectionHasOccurred ...
			selectedFolderResourceURIs = [];
		}
        return selectedFolderResourceURIs;
    },

    refresh: function(refreshCompletedCallback) {
        this._startRefresh(refreshCompletedCallback);
    },

    showAsBusy: function(isBusy) {
        if (this._tree) {
            var cursorToUse = (isBusy ? "progress" : "")
            this._tree.domNode.style.cursor = cursorToUse;
            // TODO Unfortunately setting the cursor for the this._tree.domNode
            // doesn't have any effect if the tree is already present.  Instead
            // we must explicitly set the cursor for every "dijitTreeContent"
            // div found within the tree.  There might be a better approach to
            // use, but we'll use this workaround for now ...
            var nodes = this._tree.domNode.getElementsByTagName("div");
            for (var i = 0; i < nodes.length; i++) {
                var node = nodes[i];
                if (node.className === "dijitTreeContent") {
                    node.style.cursor = cursorToUse;
                }
            }
        }
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

    // Superclass override functions ...

    _populateContents: function() {
        // Note: this is an override of a superclass function ...
		this.showLoadingMessage();
        var jsonFolderStore = {
            "identifier": JSON_PERMALINK_PROP,
            "label": JSON_NAME_PROP,
            "items": []
        };
        var folderStore = new com.ibm.rdm.web.project._FolderStore({
            projectURI: this.projectURI,
            data: jsonFolderStore
            }, null
        );
        var self = this;
		var showToplevelFolders = function() {
			// Expand the root node, so that toplevel folders under root are visible.
			var rootNode = self._tree.rootNode;
			if (!rootNode.isExpanded) {
				self._tree._onExpandoClick({node:rootNode});
			}
		};
		var selectRootFolder = function() {
			var rootNode = self._tree.rootNode;
			self._selectNode(rootNode.item, rootNode);
			self._initialRootSelectionHasOccurred = true;
		};
        var onItem = function() {
        	var treeModel = new dijit.tree.TreeStoreModel({
        		 store: folderStore,
                 query: { type: JSON_TYPE_VALUES.folder },
                 labelAttr: JSON_NAME_PROP,
                 typeAttr: JSON_TYPE_PROP
			});
            self._tree = new dijit.Tree(
                {
                	model: treeModel
                }
            );
            self._contentAreaDiv.innerHTML = "";
            self._contentAreaDiv.appendChild(self._tree.domNode);
            var onClickHandle = self.connect(self._tree, "onClick", self._treeNodeClicked);
			var onCloseHandle = self.connect(self._tree, "onClose", self._treeNodeClosed);
            self._connectHandles = [onClickHandle, onCloseHandle];
            if (self._refreshInfo) {
				var treeNodeToSelect = self._tree._itemNodeMap[self._refreshInfo.selectedFolderURI];
                if (treeNodeToSelect) {
                    var onComplete = function() {
                        self._endRefresh();
                    }
                    if (treeNodeToSelect) {
                        self._selectNodeOrBranch(treeNodeToSelect.item, treeNodeToSelect, onComplete);
                    } else {
						self._endRefresh();
                    }
                } else if (self._refreshInfo.selectedFolderURI) {
                    var expandNodeHandle = self.connect(self._tree, "_expandNode", self._expandNode);
                    self._connectHandles.push(expandNodeHandle);
                    self._expandNodeHandle = expandNodeHandle;
                } else {
					self._endRefresh();
				}
            } else {
				selectRootFolder();
			}
            self._tree.startup();
			showToplevelFolders();
        };
        var onError = function() {
            // TODO Implement
        };
        this._folderStore = folderStore;
        this._folderStore.loadToplevelItems({
            onItem: onItem,
            onError: onError,
            scope: this
        });
    },

    _handleClearFilters: function() {
        // Note: this is an override of a superclass function ...
        this._removeSelection();
    },

	_isAtLeastOneFilterSpecified: function() {
        // Note: this is an override of a superclass function ...
    	if(this._enableClearFiltersOnStart){
    		this._enableClearFiltersOnStart=false;
    	    return true;
    	}
    	
		return (this.getSelectedFolders().length > 0);
	},

    // Normal (non-override) functions ...

    _expandNode: function(treeNode) {
        // summary:
        //    This function is called whenever tree nodes are expanded.  We're
        //    only interested in "listening" for expansions when the dojo tree
        //    control's cookie information is parsed to open the last set of
        //    opened nodes.  The reason we're interested in this is that unless
        //    we wait for this to occur, our programmatic setting of a selected
        //    node (to handle the case of the user clicking on Refresh) would
        //    not work.
        if (this._refreshInfo) {
            var selectedFolderURI = this._refreshInfo.selectedFolderURI;
            var treeNodeToSelect = null;
            if (selectedFolderURI === this._getItemPermalink(treeNode.item)) {
                treeNodeToSelect = treeNode;
            } else {
                var children = treeNode.getChildren();
                for (var i = 0; i < children.length; i++) {
                    var childTreeNode = children[i];
                    if (selectedFolderURI === this._getItemPermalink(childTreeNode.item)) {
                        treeNodeToSelect = childTreeNode;
                        break;
                    }
                }
            }
            if (treeNodeToSelect) {
                var self = this;
                var onComplete = function() {
                    self.disconnect(this._expandNodeHandle);
                    // Remove the _expandNodeHandle from _connectHandles.
                    for (var i = 0; i < self._connectHandles.length; i++) {
                        var handle = self._connectHandles[i];
                        if (handle == this._expandNodeHandle) {
                            self._connectHandles.splice(i, 1);
                        }
                    }
                    self._expandNodeHandle = null;
                    self._endRefresh();
                }
                this._selectNodeOrBranch(treeNodeToSelect.item, treeNodeToSelect, onComplete);
            }
        }
    },

    _startRefresh: function(refreshCompletedCallback) {
        // To handle the case of a refresh being asked for before the last one
        // has completed, we'll ignore subsequent requests if one is currently
        // under way.
        if (!this._refreshInfo) {
            var selectedFolderURI = (
                this._selectionInfo.dataItem ?
                this._getItemPermalink(this._selectionInfo.dataItem) : null);
            var parentFolderURIs = [];
            if (selectedFolderURI) {
     
                var parentItemPermalink = this._getItemParent(this._selectionInfo.dataItem);
                if (parentItemPermalink) {
                    var parentItem = this._folderStore.getLoadedItem(parentItemPermalink);
                    do {
                        if (parentItem) {
                            parentFolderURIs.push(parentItemPermalink);
                        }
                        parentItemPermalink = parentItem.parent[0];
                        parentItem = this._folderStore.getLoadedItem(parentItemPermalink);
                    } while (parentItem != null);
                }
                parentFolderURIs = parentFolderURIs.reverse();
            }
            this._removeSelection(true);
            if (this._connectHandles) {
                for (var i = 0; i < this._connectHandles.length; i++) {
                    this.disconnect(this._connectHandles[i]);
                }
            }
            this._tree.destroyRecursive();
            delete(this._folderStore);
        
        //Perform a quick HEAD on an existing folder to get around 22170 (if a selected
        //folder is deleted and the interface is refreshed, UI hangs until browser refresh)
            
        if (selectedFolderURI){
            var selectedFolderDeferred = RDMClient.doHead({
        		uri: selectedFolderURI
        	});
            
            selectedFolderDeferred.addCallback(this, function(selectedFolderHeadResponse){
            		this._refreshInfo = {
            			selectedFolderURI: selectedFolderURI,
            			parentFolderURIs: parentFolderURIs,
            			refreshCompletedCallback: refreshCompletedCallback
            		};
            		this._populateContents();
            	});

            selectedFolderDeferred.addErrback(this, function(selectedFolderHeadResponse){
            		this._refreshInfo = {
            			selectedFolderURI: null,
            			parentFolderURIs: parentFolderURIs,
            			refreshCompletedCallback: refreshCompletedCallback
            		};
            		this._populateContents();
				});
            }
        else{
        	this._refreshInfo = {
        			selectedFolderURI: null,
        			parentFolderURIs: parentFolderURIs,
        			refreshCompletedCallback: refreshCompletedCallback
        		};
        	this._populateContents();
        }
      }
    },

    _endRefresh: function() {
        if (this._refreshInfo) {
            var callback = this._refreshInfo.refreshCompletedCallback;
            this._refreshInfo = null;
            callback();
        }
    },

    _getItemPermalink: function(dataItem) {
        return dataItem[JSON_PERMALINK_PROP][0];
    },

    _getItemParent: function(dataItem) {
        var parentValue = dataItem[JSON_PARENT_PROP];
        return (parentValue ? parentValue[0] : null);
    },

    _appendDescendantFolders: function(dataItem, descendantFolderResourceURIs) {
        // summary:
        //     Fill descendantFolderResourceURIs with the resource URIs for
        //     the folder represented by dataItem, as well as all descendant
        //     subfolders (regardless of their depth).
        // note:
        //     The _selectBranch function has already been called to make sure
        //     that all folders under the folder associated with dataItem have
        //     already been fully loaded.
        var permalink = this._folderStore.getValue(dataItem, JSON_PERMALINK_PROP);
        descendantFolderResourceURIs.push(permalink);
        var children = this._folderStore.getValues(dataItem, JSON_CHILDREN_PROP);
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            // Recurse ...
            this._appendDescendantFolders(child, descendantFolderResourceURIs);
        }
        return descendantFolderResourceURIs;
    },

	_selectNodeOrBranch: function(dataItem, treeNodeToSelect, onComplete) {
		if (this._SELECTION_INCLUDES_CHILD_FOLDERS) {
			this._selectBranch(dataItem, treeNodeToSelect, onComplete);
		} else {
			this._selectNode(dataItem, treeNodeToSelect, onComplete);
		}
	},

    _treeNodeClicked: function(/* dojo.data */ item, /*TreeNode*/ node) {
		// summary:
		//    Called when a tree node has been clicked on OR when a currently
		//    unselected tree node has been collapsed.
		if (this._selectionInfo.treeNode === node) {
			// The user has clicked on a tree node that is already selected.
			// Clear the selection for this tree node.
			if(this.isFilterDeselectionByFolderAllowed)
				this._removeSelection();
		} else {
			// The user has selected a tree node that is not currently selected.
			// Move selection to this tree node.
			if (this._SELECTION_INCLUDES_CHILD_FOLDERS) {
				this._selectBranch(item, node);
			} else {
				this._selectNode(item, node);
			}
		}
    },

	_treeNodeClosed: function(/* dojo.data */ item, /*TreeNode*/ node) {
		// summary:
		//    When a tree node is closed (i.e. collapsed) this function is
		//    called to select this newly collapsed node and update the
		//    result set appropriately.
		if (this._selectionInfo.treeNode != node) {
			// If we just collapsed a tree node AND the selected node is NOT
			// the same node as this collapsed node THEN select the contents
			// of this newly collapsed tree node.
			this._treeNodeClicked(item, node);
		}
	},

	_selectNode: function(dataItem, treeNode, onComplete) {
		// summary:
		//    Select a single folder.  After changing the folder's label to
		//    appear as selected, we then ask the dashboard to update the
		//    results to include this new folder selection criteria.
        if (this._selectionInfo.treeNode) {
            dojo.removeClass(this._selectionInfo.treeNode.labelNode, "select-folder-filter-node");
        }
        this._selectionInfo.treeNode = treeNode;
        this._selectionInfo.dataItem = dataItem;
        dojo.addClass(this._selectionInfo.treeNode.labelNode, "select-folder-filter-node");
		this._folderSelectionHasChanged();
		if (onComplete) {
			onComplete();
		}
    },

    _selectBranch: function(dataItem, treeNode, onComplete) {
		// summary:
		//    Select a folder as well as all descendant folders.  To "select"
		//    the folder and the descendants we muck with the div containing
		//    this folder so that the folder as well as all descendant folders
		//    appear as selected.  As a last step we ask that the dashboard
		//    perform an update to include this new folder selection criteria.
        if (this._selectionInfo.treeNode) {
            dojo.removeClass(this._selectionInfo.treeNode.domNode, "select-folder-and-descendants-filter-node");
        }
        this._selectionInfo.treeNode = treeNode;
        this._selectionInfo.dataItem = dataItem;
        dojo.addClass(this._selectionInfo.treeNode.domNode, "select-folder-and-descendants-filter-node");
        var onItem = function() {
            this._folderSelectionHasChanged();
            if (onComplete) {
                onComplete();
            }
        }
        this._folderStore.loadDescendantItems({
            item: dataItem,
            onItem: onItem,
            onError: onItem,
            scope: this
        });
    },

    _removeSelection: function(doSuppressInformingDashboard) {
        if (this._selectionInfo.treeNode) {
			if (this._SELECTION_INCLUDES_CHILD_FOLDERS) {
				dojo.removeClass(this._selectionInfo.treeNode.domNode, "select-folder-and-descendants-filter-node");
			} else {
				dojo.removeClass(this._selectionInfo.treeNode.labelNode, "select-folder-filter-node");
			}
        }
        this._selectionInfo.treeNode = null;
        this._selectionInfo.dataItem = null;
        if (!doSuppressInformingDashboard) {
            this._folderSelectionHasChanged();
        }
    },

    _folderSelectionHasChanged: function() {
		this._filterSelectionHasChanged();
		if (this._initialRootSelectionHasOccurred && this._refreshInfo === null) {
			this.dashboardTabPage.refresh();
		}
    }
});



/**
 * Implements a read store for lazily loading folders used by the FoldersSection.
 */
dojo.declare("com.ibm.rdm.web.project._FolderStore", dojo.data.ItemFileReadStore, {

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields

    projectURI: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields

    // TODO Turn this into a property object ... mapping folderURIs to Integers
    // representing the numOutstandingLoadRequests (to handle the case of a user
    // selecting a different branch before the currently selected branch has
    // fully loaded).
    _numOutstandingLoadRequests: null,
    _mapIdentityToItem: null,
    _mapFolderURIToChildren: null,

    _beingDestroyed: false,
    _deferredArray: null,
    _foldersDeferred: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    constructor: function(/* object */ keywordParameters) {
        this.projectURI = keywordParameters.projectURI;
        this._mapIdentityToItem = {};
        this._beingDestroyed = false;
        this._deferredArray = [];
    },
    
    destroy: function() {
        this._beingDestroyed = true;
        if (this._deferredArray) {
            for (var i = 0; i < this._deferredArray.length; i++) {
                var deferred = this._deferredArray[i];
                if (deferred.fired == -1) {
                    deferred.cancel();
                }
            }
        }
    },

    // Note: This function is based on a function by the same name found
    // in dojo's LazyLoadJSIStore.js file.
    isItemLoaded: function(/*object*/ item) {
        // summary:
        //    Overload of the isItemLoaded function to look for items of type 'stub',
        //    which indicate the data hasn't been loaded in yet.
        if (this.getValue(item, "type") === JSON_TYPE_VALUES.stub){
            return false;
        }
        return true;
    },

    // Note: This function is loosely based on a function by the same name found
    // in dojo's LazyLoadJSIStore.js file.
    loadItem: function(keywordArgs) {
        // summary:
        //    Overload of the loadItem function to demand load an item.
        var item = keywordArgs.item;
        this._assertIsItem(item);
        var self = this;
        var permalink = this.getValue(item, JSON_PERMALINK_PROP);
        var changeItemType = function(item, newTypeValue) {
            delete item[JSON_TYPE_PROP];
            item[JSON_TYPE_PROP] = newTypeValue;
        };
        var addNewStubChild = function(childFolderName, childPermalink) {
            var stub = self._createStub(childFolderName, childPermalink, permalink);
            var children = self.getValues(item, JSON_CHILDREN_PROP);
            self._arrayOfAllItems.push(stub);
            stub[self._storeRefPropName] = self;
            stub[self._itemNumPropName] = (self._arrayOfAllItems.length - 1);
            children.push(stub);
        };
        var handleQuerySuccessResponse = function(response) {
            var entries = response.getElementsByTagName("entry");
            changeItemType(item, JSON_TYPE_VALUES.folder);
            self._mapIdentityToItem[permalink] = item;
            for (var i = 0; i < entries.length; i++) {
                var entry = entries[i];
                var childFolderName = queryutils.getTitle(entry);
                var childPermalink = queryutils.getResourceURI(entry);
                addNewStubChild(childFolderName, childPermalink);
            }
            self._invokeOnItemCallbackIfNeeded(keywordArgs, item);
            return response;
        };
        var handleQueryErrorResponse = function(error) {
            changeItemType(item, JSON_TYPE_VALUES.stub);
            self._invokeOnErrorCallbackIfNeeded(keywordArgs, item);
        };

        changeItemType(item, JSON_TYPE_VALUES.folder);
        this._mapIdentityToItem[permalink] = item;
        var folders = this._mapFolderURIToChildren[permalink];
        if (folders) {
        	folders.sort(com.ibm.rdm.web.project.FoldersSection.nameComparator);
	        for (var i = 0; i < folders.length; i++) {
	        	addNewStubChild(folders[i].title, folders[i].resourceURI);
	        }
        }
        this._invokeOnItemCallbackIfNeeded(keywordArgs, item);
    },

    getLoadedItem: function(permalink) {
        // summary:
        //    Not part of Read API ... get the item that has already been
        //    loaded using a particular permalink.
        return this._mapIdentityToItem[permalink];
    },

    loadToplevelItems: function(keywordArgs) {
        // summary:
        //    Not part of Read API ... this function is called to preload all
        //    of our toplevel folders.  All folders found beneath these folders
        //    will subsequently be loaded lazily (as the user expands
        //    previously unexpanded folders) or on a demand load basis (as the
        //    user selects folders [in which case the loadDescendantItems
        //    function will be called every time folder selection changes]).
        var self = this;
        var numOutstandingLoadRequests = 0;
        var atLeastOneLoadRequestFailed;
        var lastError = null;
        var invokeCallbackIfNeeded = function(item) {
            if (numOutstandingLoadRequests == 0) {
                if (atLeastOneLoadRequestFailed) {
                    self._invokeOnErrorCallbackIfNeeded(keywordArgs, lastError);
                } else {
                    self._invokeOnItemCallbackIfNeeded(keywordArgs, item);
                }
            }
        };
        var onItem = function(item) {
            numOutstandingLoadRequests--;
            invokeCallbackIfNeeded(item);
        };
        var onError = function(error) {
            numOutstandingLoadRequests--;
            atLeastOneLoadRequestFailed = true;
            self.lastError = error;
            invokeCallbackIfNeeded();
        };
        var addRootFolder = function(rootFolderName, toplevelPermalink) {
            var stub = self._createStub(rootFolderName, toplevelPermalink);
            self._arrayOfTopLevelItems.push(stub);
            self._arrayOfAllItems.push(stub);
            stub[self._storeRefPropName] = self;
            stub[self._itemNumPropName] = (self._arrayOfAllItems.length - 1);
            stub[this._rootItemPropName] = true;
			stub[JSON_TYPE_PROP] = [JSON_TYPE_VALUES.folder];
            self._jsonData.items.push(stub);
			return stub;
        };
        var addToplevelFolder = function(toplevelFolderName, toplevelPermalink) {
            var stub = self._createStub(toplevelFolderName, toplevelPermalink);
            self._arrayOfAllItems.push(stub);
            stub[self._storeRefPropName] = self;
            stub[self._itemNumPropName] = (self._arrayOfAllItems.length - 1);
            stub[this._rootItemPropName] = false;
			return stub;
        };
        var handleQuerySuccessResponse = function(response) {
            this._mapFolderURIToChildren = response;
            var rootFolders = this._mapFolderURIToChildren[com.ibm.rdm.web.common.PublicTagsParser.ROOT];
            numOutstandingLoadRequests = rootFolders.length;
            rootFolders.sort(com.ibm.rdm.web.project.FoldersSection.nameComparator);
            
            var deferred = ProjectCache.getProject(this.projectURI);
            deferred.addCallback(this, function(project){
            	try {
	            	var stub = addRootFolder(project.name, this._mapFolderURIToChildren["top"][0].resourceURI);
	    			var children = this.getValues(stub, JSON_CHILDREN_PROP);
	                for (var i = 0; i < rootFolders.length; i++) {
	                	var childStub = addToplevelFolder(rootFolders[i].title, rootFolders[i].resourceURI);
	    				children.push(childStub);
	                }
	     			this._invokeOnItemCallbackIfNeeded(keywordArgs, stub);
            	}
				catch(e) {
	        		console.error("Error parsing folders feed: " + e);
	        	}
            });	
        };
        var handleQueryErrorResponse = function(error) {
            this._invokeOnErrorCallbackIfNeeded(keywordArgs, error);
        };

        this._foldersDeferred = com.ibm.rdm.web.common.ProjectCache.getProjectFolders(this.projectURI);
        this._foldersDeferred.addCallback(this, handleQuerySuccessResponse);
    },

    loadDescendantItems: function(keywordArgs) {
        // summary:
        //    Not part of Read API ... this function is called to fully load all
        //    descendant folders that live underneath a particular folder.
        //    We need this to support the "additive" notion that the UI folks
        //    want when selection occurs on a folder: selecting a folder is
        //    supposed show all direct descendant artifacts AS WELL AS the
        //    direct descendant artifacts for all subfolders (regardless of
        //    their depth below the selected folder).
        var self = this;
        var item = keywordArgs.item;
        var depth = (keywordArgs.depth ? keywordArgs.depth : 0);
        if (depth == 0) {
            this._numOutstandingLoadRequests = 0;
        }
        var invokeCallbackIfNeeded = function(item) {
            if (self._numOutstandingLoadRequests == 0) {
                self._invokeOnItemCallbackIfNeeded(keywordArgs, item);
            }
        };
        var onItem = function(item) {
            self._numOutstandingLoadRequests--;
            loadChildItemsAsNeeded(item);
            invokeCallbackIfNeeded();
        };
        var onError = function(error) {
            self._numOutstandingLoadRequests--;
            invokeCallbackIfNeeded();
        };
        var loadChildItemsAsNeeded = function(item) {
            var children = self.getValues(item, JSON_CHILDREN_PROP);
            for (var i = 0; i < children.length; i++) {
                var child = children[i];
                if (!self.isItemLoaded(child)) {
                    self._numOutstandingLoadRequests++;
                    self.loadItem({
                        item: child,
                        onItem: onItem,
                        onError: onError
                    });
                } else {
                    // Recurse ...
                    self.loadDescendantItems({
                        item: child,
                        onItem: keywordArgs.onItem,
                        onError: keywordArgs.onError,
                        scope: keywordArgs.scope,
                        depth: depth+1
                    });
                }
            }
        }
        loadChildItemsAsNeeded(item);
        if (this._numOutstandingLoadRequests == 0 && depth == 0) {
            invokeCallbackIfNeeded(item);
        }
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

    _createStub: function(name, permalink, parent) {
        var stub = {};
        stub[JSON_NAME_PROP] = [name];
        stub[JSON_TYPE_PROP] = [JSON_TYPE_VALUES.stub];
        stub[JSON_PERMALINK_PROP] = [permalink];
        stub[JSON_PARENT_PROP] = (parent ? [parent] : []);
        stub[JSON_CHILDREN_PROP] = [];
        return stub;
    },

    _invokeOnItemCallbackIfNeeded: function(keywordArgs, item) {
        if (keywordArgs.onItem) {
            var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
            keywordArgs.onItem.call(scope, item);
        }
    },

    _invokeOnErrorCallbackIfNeeded: function(keywordArgs, error) {
        if (keywordArgs.onError) {
            var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
            keywordArgs.onError.call(scope, error);
        }
    }
});

com.ibm.rdm.web.project.FoldersSection.nameComparator = function(tag1, tag2) {
    var resourceName1 = tag1.title;
    var resourceName2 = tag2.title;
    return ((resourceName1 == resourceName2) ? 0 : 
    	((resourceName1.toLowerCase() < resourceName2.toLowerCase()) ? -1 : 1));
};

})();
