/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ArtifactTableDescriptionSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.reviewTypes");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;


dojo.declare(
	"com.ibm.rdm.web.review.ArtifactTableDescriptionSection",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Provides a widget used to display "capture" information about the
	//    artifacts found within the review.
	// description:
	//    This widget is placed above artifact tables found within a review
	//    to show "capture" information about the artifacts.  This information
	//    includes such things as: whether the artifacts are "live" or pinned
	//    to a particular version (i.e. from "informal" or "formal" review);
	//    if a collection is involved that's not pinned to a snapshot, the
	//    collection name as well as the collection capture time; if a
	//    collection is involved that's pinned to a snapshot, the name of the
	//    collection as well as the capture time for the collection and the
	//    capture time for the snapshot.

	templateString: "<div class=\"rdmArtifactTableDescriptionSection\"></div>",
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	reviewParser: null,
	doUseAbbreviatedDisplay: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	// _connectHandles: Array
	//    When the artifacts are based on a collection, a hyperlink for this
	//    collection will be shown.  The onclick handler for this hyperlink
	//    will be added to this array so that it may be propertly removed in
	//    when callers reset this widget's reviewParser field.
	_connectHandles: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._connectHandles = [];
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	startup: function() {
		this._updateContents();
	},

	destroy: function() {
		this._removeConnectHandles();
		this.inherited("destroy", arguments);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		this.reviewParser = reviewParser;
		this._updateContents();
	},

	_updateContents: function() {
		var collectionName = this.reviewParser.getCollectionName();
		var collectionURI = this.reviewParser.getCollectionUri();
		if (collectionURI) {
			this._updateContentsCollectionSpecified(collectionURI, collectionName);
		} else {
			this._updateContentsNoCollectionSpecified();
		}
	},

	_updateContentsNoCollectionSpecified: function() {
		if (this.reviewParser.isFormal()) {
			// We only display capture information if the review has moved past
			// the Draft state ...
			if (this.reviewParser.getReviewState()
				!== reviewTypes.REVIEW_STATE_TYPES.Draft
				) {
				// Artifacts within the review are pinned to particular versions.
				var artifactsCapturedDate = this.reviewParser.getStartTimestampDate();
				var artifactsCapturedDateAsString = dojo.date.locale.format(
					artifactsCapturedDate,
		 			{ formatLength: "medium" }
				);
				var artifactsCapturedOnLabel = dojo.string.substitute(
					this._messages["artifactsCapturedOn"],
					[artifactsCapturedDateAsString]
				);
				this.domNode.innerHTML = (
					"<span>" + artifactsCapturedOnLabel + "</span>"
				);
			}
		} else {
			// Artifacts within the review are "live" (i.e. use current HEAD
			// version).
			this.domNode.innerHTML = (
				"<span>" + this._messages["artifactVersionsAreLive"] + "</span>"
			);
		}
	},

	_updateContentsCollectionSpecified: function(collectionURI, collectionName) {
		var artifactsCapturedDate = this.reviewParser.getStartTimestampDate();
		var artifactsCapturedDateAsString = dojo.date.locale.format(
			artifactsCapturedDate, { formatLength: "medium" }
		);
		var doShowArtifactCaptureInfo =
			(this.reviewParser.getReviewState() !== reviewTypes.REVIEW_STATE_TYPES.Draft);
		if (this.reviewParser.isFormal()) {
			// Artifacts within the review are pinned to particular versions.
			if (this.doUseAbbreviatedDisplay) {
				// Use abbreviated display.
				var artifactsCapturedOnLabel = dojo.string.substitute(
					this._messages["artifactsCapturedOn"],
					[artifactsCapturedDateAsString]
				);
				this.domNode.innerHTML = (
					"<span>" + artifactsCapturedOnLabel + "</span>"
				);
			} else {
				// Use verbose display.
				var snapshotDate = this.reviewParser.getCollectionSnapshotDate();
				var snapshotDateAsString = (
					snapshotDate ?
					dojo.date.locale.format(artifactsCapturedDate, { formatLength: "medium" }) :
					null
				);
				var artifactsCapturedOnLabel = (
				    snapshotDate ?
					dojo.string.substitute(
						this._messages["artifactsCapturedOnAsDefinedByCollection"],
						[snapshotDateAsString]) :
					dojo.string.substitute(
						this._messages["collectionAndArtifactsCapturedOn"],
						[artifactsCapturedDateAsString])
				);
				this.domNode.innerHTML = (
					"<span>" + this._messages["artifactListDefinedBy"] + ":&nbsp;</span>"
					+ (doShowArtifactCaptureInfo ?
						"<br><span>" + artifactsCapturedOnLabel + "</span>" :
						"")
				);
				var collectionAnchor =
					ResourceLinksHandler.getResourceLink(
						collectionURI, collectionName, true, true);
				dojo.place(collectionAnchor, this.domNode.firstChild, "after");
				this._connectHandles.push(dojo.connect(collectionAnchor, "onclick", function(event){
			    	ResourceLinksHandler.handleAnchorClicked(event);
				}));
			}
		} else {
			// Artifacts within the review are "live" (i.e. use current HEAD
			// version).
			if (this.doUseAbbreviatedDisplay) {
				// Use abbreviated display.
				this.domNode.innerHTML = (
					"<span>" + this._messages["artifactVersionsAreLive"] + "</span>"
				);
			} else {
				// Use verbose display.
				this.domNode.innerHTML = (
					"<span>" + this._messages["artifactListDefinedBy"] + ":&nbsp;</span>"
					+ "<br><span>" + this._messages["artifactVersionsAreLive"] + "</span>"
				);
				var collectionAnchor =
					ResourceLinksHandler.getResourceLink(
						collectionURI, collectionName, true, true);
				dojo.place(collectionAnchor, this.domNode.firstChild, "after");
				this._connectHandles.push(dojo.connect(collectionAnchor, "onclick", function(event){
			    	ResourceLinksHandler.handleAnchorClicked(event);
				}));
			}
		}
	},

	_removeConnectHandles: function() {
		if (this._connectHandles) {
			for (var i = 0; i < this._connectHandles.length; i++) {
				dojo.disconnect(this._connectHandles[i]);
			}
		}
		this._connectHandles = [];
	}
});


})();
