/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewCache");

dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.ParticipantResultsParser");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var Headers = com.ibm.rdm.web.server.Headers;
var ParticipantResultsParser = com.ibm.rdm.web.review.ParticipantResultsParser;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var UpdateContext = com.ibm.rdm.web.common.UpdateContext;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;


com.ibm.rdm.web.review.ReviewCache = {
	// summary:
	//    This singleton is used to cache Review related information.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_mapResourceURIToETag: null,
	_mapResourceURIToReviewParser: null,
	_lastUpdateContext: null,

	_participantResultsHeadInfo: {
		updateContext: null,
		mapResourceURIToHeadDeferred: null,
		mapResourceURIToHeadCallbackResponse: null,
		mapResourceURIToHeadErrbackResponse: null
	},
	
	///////////////////////////////////////////////////////////////////////////
	// API

	startup: function() {
		this._mapResourceURIToETag = new Object();
		this._mapResourceURIToReviewParser = new Object();
		this._lastUpdateContext = null;
	},

	setReview: function(resourceURI, revision, reviewParser) {
		// summary:
		//    This function is called to make an initial resourceURI to
		//    reviewParser mapping.
		// description:
		//    Please note that ONLY the ApprovalEditor should make use of
		//    this function, as all other code should just call this class's
		//    getReview function to obtain a fully populated reviewParser.
		this._mapResourceURIToETag[resourceURI] = revision;
		this._mapResourceURIToReviewParser[resourceURI] = reviewParser;
		this._lastUpdateContext = UpdateContext.getValue();
	},

	getReview: function(reviewId, /*Boolean?*/doExpectVotesFromMe) {
		// summary:
		//    Get the current version a particular review.
		// returns:
		//    A deferred which you can to obtain the most up to date information
		//    found within the Review (i.e. the Review represented by the passed
		//    "reviewId").
		//    Please note that the passed response to your callback is of a
		//    similar format to that used by the RDMClient.doGet function:
		//    {
		//        doc: <an instance of a ReviewParser>
		//        ioArgs: <the original ioArgs that was used in the GET>
		//    }
		//    The only difference is that the "doc" property is an instance of a
		//    ReviewParser rather than an instance of an XML document (as it
		//    would be with the RDMClient.doGet function).
		// example:
		//    |    var onReviewLoaded = function(response) {
		//    |        var reviewParser = response.doc;
		//    |        self._artifactBanner.attr("reviewParser", reviewParser);
		//    |        self._artifactBanner.startup();
		//    |    };
		//    |    var deferred = com.ibm.rdm.web.review.ReviewCache.getReview(this.reviewId);
		//    |    deferred.addCallback(onReviewLoaded);
		// reviewId: String
		//    A string such as "_lckswDjhEd67TdjgK4pk9g".
		// doExpectVotesFromMe: Boolean
		//    This is passed as true whenever votes have been cast (or Done has
		//    been pressed).  It's passed to avoid server defect 19081 (by using
		//    a workaround implemented via defect 19765).
		//
		var self = this;
		var resourceURI = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + reviewId;
		var projectURI = null;
		var reviewParser = null;
		var participantResultsParser = null;
		var errorOccurred = false;
		var ioArgs = null;
		// Note: The logic here is a little involved.  In the worst case we
		// need to make sure that 3 deferreds have been completed before
		// calling back deferred that this function returns.  This is due
		// to the fact that in order to appropriately populate a ReviewParser
		// a GET for Review and a GET for associated ParticipantResults must
		// be completed as well as (potentially) a GET for describe results for
		// the Review (in order to know what projectURI the Review lives in).
		var numOutstandingDeferreds = 2;
		// Here's the manufactured deferred that will be returned by this
		// function (and that will eventually be invoked with a call to its
		// callback or errback as appropriate [by one of the deferreds
		// found in this function]).
		var reviewArtifactsCompletelyLoadedDeferred = new dojo.Deferred();
		var combineResultsAndInvokeCallbackIfNeeded = function() {
			// Preconditions: The following vars now contain non-null values:
			// reviewParser and projectURI.  Also, if the review is not in the
			// Draft state, participantResultsParser will be non-null as well.
			if (numOutstandingDeferreds === 0 && (! errorOccurred)) {
				if (participantResultsParser) {
					reviewParser.updateParticipantResults(participantResultsParser);
				}
				reviewParser.setProjectUri(projectURI);
				var participantResultResourceURI = reviewParser.getParticipantResultUri();
				if (participantResultResourceURI) {
					var headDeferred =
						self.getParticipantResultHeadDeferred(participantResultResourceURI, projectURI);
					headDeferred.addCallback(this, function(response){
						reviewParser.setParticipantResultRevision(XhrUtils.getResponseEtag(response));
						reviewArtifactsCompletelyLoadedDeferred.callback(
							{ doc: reviewParser, ioArgs: ioArgs });
						return response;
					});
				} else {
					reviewArtifactsCompletelyLoadedDeferred.callback(
						{ doc: reviewParser, ioArgs: ioArgs });
				}
			}
		};
		var loadParticipantResults = function(projectURI) {
			var headers = new Object();
			var currentUpdateContext = UpdateContext.getValue();
			this._lastUpdateContext = currentUpdateContext;
			headers[Headers.OWNING_CONTEXT] = projectURI;
			var loadReviewResults = function(/*Boolean?*/doSkipCache) {
				return RDMClient.doGet({
					uri: UriHelper.getReviewParticipantResultsUri(resourceURI),
					xhrArgs: { headers: headers },
					skipCache: doSkipCache
				});
			}
			var loadParticipantResultsDeferred = loadReviewResults();
			var onParticipantResultsLoaded = function(response) {
				participantResultsParser = new ParticipantResultsParser({
					doc: response.doc
				});
				if (!self._isReviewResultsReloadNeeded(doExpectVotesFromMe, participantResultsParser)) {
					numOutstandingDeferreds--;
					combineResultsAndInvokeCallbackIfNeeded();
				} else {
					// Avoid defect 19081 by asking for the results again (as we
					// were just informed that we have no results when we EXPECT
					// results [as we've just voted successfully]).
					var onRetryParticipantResultsLoaded = function(response) {
						participantResultsParser = new ParticipantResultsParser({
							doc: response.doc
						});
						numOutstandingDeferreds--;
						ErrorHandler.warn("Needed to reissue review results query.");
						combineResultsAndInvokeCallbackIfNeeded();
						return response;
					};
					setTimeout(
						function() {
							loadParticipantResultsDeferred = loadReviewResults(true);
							loadParticipantResultsDeferred.addCallback(onRetryParticipantResultsLoaded);
							loadParticipantResultsDeferred.addErrback(onParticipantResultsError);
						},
						2000);
				}
				return response;
			};
			var onParticipantResultsError = function(response) {
				errorOccurred = true;
				reviewArtifactsCompletelyLoadedDeferred.errback(response);
				return response;
			};
			loadParticipantResultsDeferred.addCallback(onParticipantResultsLoaded);
			loadParticipantResultsDeferred.addErrback(onParticipantResultsError);
		};
		var loadReview = function(projectURI) {
			var onReviewLoaded = function(response) {
				var etag = XhrUtils.getResponseEtag(response);
				ioArgs = response.ioArgs;
				if (self._mapResourceURIToETag[resourceURI] !== etag) {
					self._mapResourceURIToETag[resourceURI] = etag;
					reviewParser = new ReviewParser({
						response: response
					});
					self._mapResourceURIToReviewParser[resourceURI] = reviewParser;
					numOutstandingDeferreds--;
					combineResultsAndInvokeCallbackIfNeeded();
				} else {
					reviewParser = self._mapResourceURIToReviewParser[resourceURI];
					numOutstandingDeferreds--;
					combineResultsAndInvokeCallbackIfNeeded();
				}
				return {
					doc: reviewParser,
					ioArgs: response.ioArgs
				};
			};
			var onReviewError = function(response) {
				errorOccurred = true;
				reviewArtifactsCompletelyLoadedDeferred.errback(response);
				return response;
			};
			var headers = new Object();
			headers[Headers.OWNING_CONTEXT] = projectURI;
			var loadReviewDeferred = RDMClient.doGet({
				uri: resourceURI,
				xhrArgs: { headers: headers }
			});
			loadReviewDeferred.addCallback(onReviewLoaded);
			loadReviewDeferred.addErrback(onReviewError);
		};
		var lastReviewParser = self._mapResourceURIToReviewParser[resourceURI];
		if (lastReviewParser
			&& lastReviewParser.getProjectUri()
			&& (lastReviewParser.getReviewState() !== reviewTypes.REVIEW_STATE_TYPES.Draft)
			) {
			// We know we can obtain the appropriate projectURI without querying
			// the server for this ... AND we know that the review is not in the
			// Draft state which means we can safely ask for vote casting
			// results.
			projectURI = lastReviewParser.getProjectUri();
			loadReview(projectURI);
			loadParticipantResults(projectURI);
		} else {
			// We either need to get at the appropriate projectURI OR we know
			// that the last version of this review was using the Draft state.
			var onDescriptionComplete = function(indexedProperties) {
				projectURI = indexedProperties.projectURI;
				loadReview(projectURI);
				// Note: We take care here to ONLY ask for vote casting results
				// if we know the review is NOT in the Draft state.
				if (indexedProperties.reviewStatus === reviewTypes.REVIEW_STATE_TYPES.Draft) {
					numOutstandingDeferreds--;
					combineResultsAndInvokeCallbackIfNeeded();
				} else {
					loadParticipantResults(projectURI);
				}
				return indexedProperties;
			};
			var deferred = ProjectCache.getResourceDescriptionMultiFetch(resourceURI);
			deferred.addCallback(onDescriptionComplete);
		}
		return reviewArtifactsCompletelyLoadedDeferred;
	},

	getParticipantResultHeadDeferred: function(participantResultResourceURI, projectURI) {
		// TODO Change RDMClient to enable is head requests with 'skipCache' so
		// that this function wouldn't be needed.
		// Alternatively -- Ask Chris to somehow provide the user's participant result
		// revision info when the user asks for the review's results (as that revision
		// info is the ONLY reason this HEAD request is made ... as we need the revision
		// info in order to pass this as the "If-Match" value when we make subsequent
		// PUT requests).
		var deferred = new dojo.Deferred();
		if (this._participantResultsHeadInfo.updateContext == null
			|| this._participantResultsHeadInfo.updateContext != UpdateContext.getValue()
			) {
			this._participantResultsHeadInfo.updateContext = UpdateContext.getValue();
			this._participantResultsHeadInfo.mapResourceURIToHeadDeferred = {};
			this._participantResultsHeadInfo.mapResourceURIToHeadCallbackResponse = {};
			this._participantResultsHeadInfo.mapResourceURIToHeadErrbackResponse = {};
		}
		var headDeferred =
			this._participantResultsHeadInfo.mapResourceURIToHeadDeferred[participantResultResourceURI];
		if (! headDeferred) {
			headDeferred = RDMClient.doHead({
				uri: participantResultResourceURI,
				projectURI: projectURI,
				skipCache: true,
				skipErrors: true
			});
			this._participantResultsHeadInfo.mapResourceURIToHeadDeferred[participantResultResourceURI] =
				headDeferred;
			headDeferred.addCallback(this, function(response) {
				this._participantResultsHeadInfo.mapResourceURIToHeadCallbackResponse[participantResultResourceURI] = response;
				deferred.callback(response);
				return response;
			});
			headDeferred.addErrback(this, function(errorResponse){
				this._participantResultsHeadInfo.mapResourceURIToHeadErrbackResponse[participantResultResourceURI] = errorResponse;
				deferred.errback(errorResponse);
				return errorResponse;
			});
		} else {
			if (headDeferred.fired == -1) {
				headDeferred.addCallback(function(results) { deferred.callback(results); return results; });
				headDeferred.addErrback(function(errorResponse) { deferred.errback(errorResponse); return errorResponse; });
			} else {
				var callbackResponse = this._participantResultsHeadInfo.mapResourceURIToHeadCallbackResponse[participantResultResourceURI];
				if (callbackResponse) {
					deferred.callback(callbackResponse);
				} else {
					deferred.errback(
						this._participantResultsHeadInfo.mapResourceURIToHeadErrbackResponse[participantResultResourceURI]);
				}
			}
		}
		return deferred;
	},

	getRevisionLastLoadedForReview: function(reviewURI) {
		return this._mapResourceURIToETag[reviewURI];
	},

	_isReviewResultsReloadNeeded: function(doExpectVotesFromMe, participantResultsParser) {
		// summary:
		//    Returns true iff either doExpectVotesFromMe is false or we can't find
		//    a vote cast by the current user in the participantResultsParser.
		// description:
		//    This is needed only to workaround the issue caused by defect 19081.
		var isReviewResultsReloadNeeded = false;
		if (doExpectVotesFromMe) {
			var participantResults = participantResultsParser.getParticipantResults();
			var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
			var myParticipantResult = participantResults[myUserURI];
			var foundVotesFromMe =
				myParticipantResult && myParticipantResult.artifactResults;
			isReviewResultsReloadNeeded = !foundVotesFromMe;
		}
		return isReviewResultsReloadNeeded;
	}
};

})();
