/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.reviewChangeNotifier");
dojo.require("com.ibm.rdm.web.review.ReviewParser");


var ReviewParser = com.ibm.rdm.web.review.ReviewParser;


var mapResourceURIToChangeHandlers = {};
var mapResourceURIToSaveInProgressHandlers = {};
var mapResourceURIToSaveHasCompletedHandlers = {};
var mapResourceURIToSaveHasFailedHandlers = {};


com.ibm.rdm.web.review.reviewChangeNotifier = {
	// summary:
	//    This singleton is used to inform interested parties when changes
	//    have occurred in review artifacts that have been opened during the
	//    current session.
	// description:
	//    This class is needed due to the fact that Review artifact changes
	//    may need to update more than one viewer whenever an update is
	//    performed.  For example, if the user does the following:
	//    - Opens a Review.
	//    - Clicks through one of the artifacts within the Review.
	//    - Changes the cast vote from Approve to Disapprove.
	//    - Hits the back button to go back to the original Review.
	//    The end result SHOULD be that the Review now shows this artifact
	//    with the newly cast vote setting of Disapprove (instead of the
	//    Approve setting that was present before the user followed the link
	//    and changed the cast vote).

	///////////////////////////////////////////////////////////////////////////
	// API

	addReviewListener: function(resourceURI,
								onReviewChanged,
								/*Callback?*/onReviewSaveIsInProgress,
								/*Callback?*/onReviewSaveHasCompleted,
								/*Callback?*/onReviewSaveHasFailed) {
		// summary:
		//    Listen for revision updates to a particular review.
		// resourceURI:
		//    The resource URI of a particular Review artifact.
		// onReviewChanged:
		//    Callback function that will be invoked whenever it's noticed that
		//    the passed resourceURI has been updated.  The callback should be
		//    of the following form:
		//    |    var onReviewChanged = function(reviewParser) {
		//    |        // Do something with reviewParser (like update UI elements)
		//    |    };
		//    |    com.ibm.rdm.web.review.reviewChangeNotifier.addReviewListener(
		//    |        this.reviewResourceURI, onReviewChanged);
		// onReviewSaveIsInProgress:
		//    Optional callback function that will be invoked whenever a new
		//    "participant results" save request has been issued which is
		//    associated with the passed review resourceURI.  This callback
		//    will pass a single argument: the ReviewParser object that was
		//    originally provided when the save was requested.
		// onReviewSaveHasCompleted:
		//    Optional callback function that will be invoked whenever an
		//    attempted "participant results" save request for the review
		//    associated with resourceURI has succeeded.  This callback
		//    will pass a single argument: the ReviewParser object that was
		//    originally provided when the save was requested.
		// onReviewSaveHasFailed:
		//    Optional callback function that will be invoked whenever an
		//    attempted "participant results" save request for the review
		//    associated with resourceURI has failed.  This callback
		//    will pass a single argument: the ReviewParser object that was
		//    originally provided when the save was requested.
		var addCallbackHandler = function(callbackHandler, mapInWhichToAddResourceCallbackHandler) {
			var callbacks = mapInWhichToAddResourceCallbackHandler[resourceURI];
			if (! callbacks) {
				callbacks = [];
				mapInWhichToAddResourceCallbackHandler[resourceURI] = callbacks;
			}
			callbacks.push(callbackHandler);
		};
		addCallbackHandler(onReviewChanged, mapResourceURIToChangeHandlers);
		if (onReviewSaveIsInProgress) {
			addCallbackHandler(onReviewSaveIsInProgress, mapResourceURIToSaveInProgressHandlers);
		}
		if (onReviewSaveHasCompleted) {
			addCallbackHandler(onReviewSaveHasCompleted, mapResourceURIToSaveHasCompletedHandlers);
		}
		if (onReviewSaveHasFailed) {
			addCallbackHandler(onReviewSaveHasFailed, mapResourceURIToSaveHasFailedHandlers);
		}
	},

	removeReviewListener: function(resourceURI,
								   onReviewChanged,
								   /*Callback?*/onReviewSaveIsInProgress,
								   /*Callback?*/onReviewSaveHasCompleted,
								   /*Callback?*/onReviewSaveHasFailed) {
		var removeCallbackHandler = function(callbackHandler, mapInWhichToRemoveResourceCallbackHandler) {
			callbacks = mapInWhichToRemoveResourceCallbackHandler[resourceURI];
			if (callbacks) {
				for (var i = 0; i < callbacks.length; i++) {
					if (callbacks[i] == callbackHandler) {
						callbacks.splice(i, 1);
					}
				}
			}
		};
		removeCallbackHandler(onReviewChanged, mapResourceURIToChangeHandlers);
		if (onReviewSaveIsInProgress) {
			removeCallbackHandler(onReviewSaveIsInProgress, mapResourceURIToSaveInProgressHandlers);
		}
		if (onReviewSaveHasCompleted) {
			removeCallbackHandler(onReviewSaveHasCompleted, mapResourceURIToSaveHasCompletedHandlers);
		}
		if (onReviewSaveHasFailed) {
			removeCallbackHandler(onReviewSaveHasFailed, mapResourceURIToSaveHasFailedHandlers);
		}
	},

	reviewHasBeenUpdated: function(reviewParser) {
		// summary:
		//    This function is called whenever a review has been successfully
		//    saved.
		// reviewParser:
		//    A ReviewParser object that contains up to date information about
		//    a particular Review artifact.
		var resourceURI = reviewParser.getSelfUri();
		var callbacks = mapResourceURIToChangeHandlers[resourceURI];
		dojo.forEach(
			callbacks,
			function(callback) { callback(reviewParser); }
		);
	},

	reviewSaveIsInProgress: function(reviewParser) {
		// summary:
		//    This function is called whenever a "save participant results"
		//    request has been issued to the server.
		// reviewParser:
		//    A ReviewParser object that contains the last loaded review
		//    information as well as the vote cast changes that the current
		//    user is attempting to save.
		var resourceURI = reviewParser.getSelfUri();
		var callbacks = mapResourceURIToSaveInProgressHandlers[resourceURI];
		dojo.forEach(
			callbacks,
			function(callback) { callback(reviewParser); }
		);
	},

	reviewSaveHasCompleted: function(reviewParser) {
		// summary:
		//    This function is called whenever a "save participant results"
		//    request has succeeded.
		// reviewParser:
		//    A ReviewParser object that contains the last loaded review as
		//    well as vote cast changes that the current user has just
		//    succeeded in saving.  In addition this ReviewParser object will
		//    also hold the current user's participant results URI (accessible
		//    via the 'getParticipantResultUri' function) and the the newly
		//    saved revision of this participant results URI (accessible via
		//    the 'getParticipantResultRevision' function).  Please note that
		//    it _may_ be the case that the participant results URI wasn't
		//    previously defined (when 'reviewSaveIsInProgress' was called)
		//    as the save _may_ have required a performing POST instead of a
		//    PUT.
		var resourceURI = reviewParser.getSelfUri();
		var callbacks = mapResourceURIToSaveHasCompletedHandlers[resourceURI];
		dojo.forEach(
			callbacks,
			function(callback) { callback(reviewParser); }
		);
	},

	reviewSaveHasFailed: function(reviewParser, errorStatus) {
		// summary:
		//    This function is called whenever a "save participant results"
		//    request has returned with an error code.
		// reviewParser:
		//    A ReviewParser object that contains the last loaded review
		//    information as well as the vote cast changes that the current
		//    user failed in attempting to save.
		var resourceURI = reviewParser.getSelfUri();
		var callbacks = mapResourceURIToSaveHasFailedHandlers[resourceURI];
		dojo.forEach(
			callbacks,
			function(callback) { callback(reviewParser, errorStatus); }
		);
	}
};

})();
