/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.reviewTypes");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var mapArtifactStatusTypeToButtonImageURI = null;
var mapArtifactStatusTypeToDisplayName = null;
var mapArtifactStatusTypeToImageURI = null;
var mapParticipantRoleTypeToDisplayName = null;
var mapParticipantRoleTypeToImageURI = null;
var mapParticipantStatusTypeToDisplayName = null;
var mapReviewStateTypeToDisplayName = null;
var mapReviewStateTypeToImageURI = null;
var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


com.ibm.rdm.web.review.reviewTypes = {
	// summary:
	//    This singleton is used to represent variable enum types used by
	//    the review and approval code.

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	// ARTIFACT_STATUS_TYPES enum
	//     Represents the status of a particular artifact for a particular
	//     participant involved in a Review.
	//     Note: The first four values must be in alignment with the values
	//     found in the native client's review.xsd file.
	ARTIFACT_STATUS_TYPES: {
		Approved: "Approved",
		Disapproved: "Disapproved",
		Abstained: "Abstained",
		Reviewed: "Reviewed",
		NotStarted: "Not Started",
		InProgress: "In Progress"
	},

	// PARTICIPANT_ROLE_TYPES enum
	//     Represents the role of a particular participant in a Review.
	//     Note: The values must be in alignment with the values found
	//     in the native client's review.xsd file.
	PARTICIPANT_ROLE_TYPES: {
		Approver: "Approver",
		RequiredReviewer: "Required Reviewer",
		OptionalReviewer: "Optional Reviewer"
	},

	// PARTICIPANT_STATUS_TYPES enum
	//     Represents the status of a particular participant involved in a
	//     Review.
	//     This value is computed (and NOT stored in the document).
	PARTICIPANT_STATUS_TYPES: {
		Started: "Started",
		NotStarted: "Not Started",
		Complete: "Complete"
	},

	// REVIEW_STATE_TYPES enum
	//     Represents the state of the Review.
	//     Note: The values must be in alignment with the values found
	//     in the native client's review.xsd file.
	REVIEW_STATE_TYPES: {
		Draft: "Draft",
		Started: "Started",
		Paused: "Paused",
		Reviewed: "Complete",
		Finalized: "Finalized"
	},

	///////////////////////////////////////////////////////////////////////////
	// API

	getArtifactStatusTypeDisplayName: function(artifactStatusType) {
		// summary:
		//    Return a string appropriate for the current locale to represent
		//    the status associated with a particular artifact.
		// artifactStatusType:
		//    One of the values found in ARTIFACT_STATUS_TYPES (e.g.
		//    ARTIFACT_STATUS_TYPES.NotStarted).
		// example:
		//    A call to:
		//    |    com.ibm.rdm.web.review.reviewTypes.getArtifactStatusTypeDisplayName(
		//    |        com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted);
		//    Would return (assuming English locale):
		//    |    "Not started"
		if (! mapArtifactStatusTypeToDisplayName) {
			this._initMapArtifactStatusTypeToDisplayName();
		}
		return mapArtifactStatusTypeToDisplayName[artifactStatusType];
	},

	getArtifactStatusTypeButtonImageURI: function(artifactStatusType) {
		// summary:
		//    Return a URI appropriate for representing a particular
		//    artifact status for a button (used for vote casting).
		// artifactStatusType:
		//    One of the values found in ARTIFACT_STATUS_TYPES (e.g.
		//    ARTIFACT_STATUS_TYPES.NotStarted).
		if (! mapArtifactStatusTypeToButtonImageURI) {
			this._initMapArtifactStatusTypeToButtonImageURI();
		}
		return mapArtifactStatusTypeToButtonImageURI[artifactStatusType];
	},

	getArtifactStatusTypeImageURI: function(artifactStatusType) {
		// summary:
		//    Return a URI appropriate for representing a particular
		//    artifact's review status.
		// artifactStatusType:
		//    One of the values found in ARTIFACT_STATUS_TYPES (e.g.
		//    ARTIFACT_STATUS_TYPES.NotStarted).
		if (! mapArtifactStatusTypeToImageURI) {
			this._initMapArtifactStatusTypeToImageURI();
		}
		return mapArtifactStatusTypeToImageURI[artifactStatusType];
	},

	getReviewStateTypeImageURI: function(reviewStateType) {
		// summary:
		//    Return a URI appropriate for representing a review's
		//    current state.
		// reviewStateType:
		//    One of the values found in REVIEW_STATE_TYPES (e.g.
		//    REVIEW_STATE_TYPES.Started).
		if (! mapReviewStateTypeToImageURI) {
			this._initMapReviewStateTypeToImageURI();
		}
		return mapReviewStateTypeToImageURI[reviewStateType];
	},

	getParticipantStatusTypeDisplayName: function(participantStatusType) {
		// summary:
		//    Return a string appropriate for the current locale to represent
		//    the status associated with a particular reviewer.
		// participantStatusType:
		//    One of the values found in PARTICIPANT_STATUS_TYPES (e.g.
		//    PARTICIPANT_STATUS_TYPES.NotStarted).
		// example:
		//    A call to:
		//    |    com.ibm.rdm.web.review.reviewTypes.getParticipantStatusTypeDisplayName(
		//    |        com.ibm.rdm.web.review.reviewTypes.PARTICIPANT_STATUS_TYPES.NotStarted);
		//    Would return (assuming English locale):
		//    |    "Not started"
		if (! mapParticipantStatusTypeToDisplayName) {
			this._initMapParticipantStatusTypeToDisplayName();
		}
		return mapParticipantStatusTypeToDisplayName[participantStatusType];
	},

	getParticipantRoleTypeDisplayName: function(participantRoleType) {
		// summary:
		//    Return a string appropriate for the current locale to represent
		//    a reviewer's role type.
		// participantRoleType:
		//    One of the values found in PARTICIPANT_ROLE_TYPES (e.g.
		//    PARTICIPANT_ROLE_TYPES.Approver).
		// example:
		//    A call to:
		//    |    com.ibm.rdm.web.review.reviewTypes.getParticipantRoleTypeDisplayName(
		//    |        com.ibm.rdm.web.review.reviewTypes.PARTICIPANT_ROLE_TYPES.Approver);
		//    Would return (assuming English locale):
		//    |    "Approver"
		if (! mapParticipantRoleTypeToDisplayName) {
			this._initMapParticipantRoleTypeToDisplayName();
		}
		return mapParticipantRoleTypeToDisplayName[participantRoleType];
	},

	getParticipantRoleTypeImageURI: function(participantRoleType) {
		// summary:
		//    Return a URI appropriate for representing a review's
		//    current state.
		// participantRoleType:
		//    One of the values found in PARTICIPANT_ROLE_TYPES (e.g.
		//    PARTICIPANT_ROLE_TYPES.Approver).
		if (! mapParticipantRoleTypeToImageURI) {
			this._initMapParticipantRoleTypeToImageURI();
		}
		return mapParticipantRoleTypeToImageURI[participantRoleType];
	},

	getReviewStateDisplayName: function(reviewStateType) {
		// summary:
		//    Return a string appropriate for the current locale to represent
		//    the state associated with a review.
		// artifactStatusType:
		//    One of the values found in REVIEW_STATE_TYPES (e.g.
		//    REVIEW_STATE_TYPES.Draft).
		// example:
		//    A call to:
		//    |    com.ibm.rdm.web.review.reviewTypes.getReviewStateDisplayName(
		//    |        com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Draft);
		//    Would return (assuming English locale):
		//    |    "Draft"
		if (! mapReviewStateTypeToDisplayName) {
			this._initMapReviewStateTypeToDisplayName();
		}
		return mapReviewStateTypeToDisplayName[reviewStateType];
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_initMapArtifactStatusTypeToDisplayName: function() {
		// summary:
		//    Initialize the mapArtifactStatusTypeToDisplayName variable.
		mapArtifactStatusTypeToDisplayName = new Object;
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.NotStarted] =
			messages["notStarted"];
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.InProgress] =
			messages["inProgress"];
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.Approved] =
			messages["approved"];
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.Disapproved] =
			messages["disapproved"];
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.Abstained] =
			messages["abstained"];
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.Reviewed] =
			messages["reviewed"];
	},

	_initMapArtifactStatusTypeToButtonImageURI: function() {
		mapArtifactStatusTypeToButtonImageURI = new Object;
		// Note: We DO NOT add icons here for NotStarted and InProgress.  This isn't
		// an oversight (the reason is that user should not be able to click on
		// any buttons that would move an artifact to either of these states).
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.Approved] =
			"com.ibm.rdm.web/review/images/icons/etool/approve.gif";
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.Disapproved] =
			"com.ibm.rdm.web/review/images/icons/etool/disapprove.gif";
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.Abstained] =
			"com.ibm.rdm.web/review/images/icons/etool/abstain.gif";
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.Reviewed] =
			"com.ibm.rdm.web/review/images/icons/etool/reviewed.gif";
	},

	_initMapArtifactStatusTypeToImageURI: function() {
		mapArtifactStatusTypeToImageURI = new Object;
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.NotStarted] =
			"com.ibm.rdm.web/review/images/icons/obj16/art-rev-not-started.gif";
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.InProgress] =
			"com.ibm.rdm.web/review/images/icons/obj16/art-rev-in-progress.gif";
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.Approved] =
			"com.ibm.rdm.web/review/images/icons/obj16/art-rev-approve.gif";
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.Disapproved] =
			"com.ibm.rdm.web/review/images/icons/obj16/art-rev-disapprove.gif";
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.Abstained] =
			"com.ibm.rdm.web/review/images/icons/obj16/art-rev-abstain.gif";
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.Reviewed] =
			"com.ibm.rdm.web/review/images/icons/obj16/art-rev-reviewed.gif";
	},

	_initMapParticipantRoleTypeToImageURI: function() {
		mapParticipantRoleTypeToImageURI = new Object;
		mapParticipantRoleTypeToImageURI[this.PARTICIPANT_ROLE_TYPES.Approver] =
			"com.ibm.rdm.web/review/images/icons/obj16/role-approver.gif";
		mapParticipantRoleTypeToImageURI[this.PARTICIPANT_ROLE_TYPES.RequiredReviewer] =
			"com.ibm.rdm.web/review/images/icons/obj16/role-reviewer.gif";
		mapParticipantRoleTypeToImageURI[this.PARTICIPANT_ROLE_TYPES.OptionalReviewer] =
			"com.ibm.rdm.web/review/images/icons/obj16/role-opt-reviewer.gif";
		// TODO Introduce "Owner" role using the following?
		// "com.ibm.rdm.web/review/images/icons/obj16/role-owner.gif"
	},

	_initMapReviewStateTypeToImageURI: function() {
		mapReviewStateTypeToImageURI = new Object;
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Draft] =
			"com.ibm.rdm.web/review/images/icons/obj16/review-draft.gif";
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Started] =
			"com.ibm.rdm.web/review/images/icons/obj16/review-started.gif";
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Paused] =
			"com.ibm.rdm.web/review/images/icons/obj16/review-paused.gif";
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Reviewed] =
			"com.ibm.rdm.web/review/images/icons/obj16/art-rev-reviewed.gif";
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Finalized] =
			"com.ibm.rdm.web/review/images/icons/obj16/review-finalized.gif";
	},

	_initMapParticipantStatusTypeToDisplayName: function() {
		// summary:
		//    Initialize the mapParticipantStatusTypeToDisplayName variable.
		mapParticipantStatusTypeToDisplayName = new Object;
		mapParticipantStatusTypeToDisplayName[this.PARTICIPANT_STATUS_TYPES.Started] =
			messages["started"];
		mapParticipantStatusTypeToDisplayName[this.PARTICIPANT_STATUS_TYPES.NotStarted] =
			messages["notStarted"];
		mapParticipantStatusTypeToDisplayName[this.PARTICIPANT_STATUS_TYPES.Complete] =
			messages["complete"];
	},

	_initMapParticipantRoleTypeToDisplayName: function() {
		// summary:
		//    Initialize the mapParticipantRoleTypeToDisplayName variable.
		mapParticipantRoleTypeToDisplayName = new Object;
		mapParticipantRoleTypeToDisplayName[this.PARTICIPANT_ROLE_TYPES.Approver] =
			messages["approver"];
		mapParticipantRoleTypeToDisplayName[this.PARTICIPANT_ROLE_TYPES.RequiredReviewer] =
			messages["reviewer"];
		mapParticipantRoleTypeToDisplayName[this.PARTICIPANT_ROLE_TYPES.OptionalReviewer] =
			messages["optionalReviewer"];
	},

	_initMapReviewStateTypeToDisplayName: function() {
		// summary:
		//    Initialize the mapReviewStateTypeToDisplayName variable.
		mapReviewStateTypeToDisplayName = new Object;
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Draft] =
			messages["draft"];
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Started] =
			messages["started"];
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Paused] =
			messages["paused"];
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Reviewed] =
			messages["reviewed"];
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Finalized] =
			messages["finalized"];
	}
};

})();
