/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.reviewUpdateMediator");

dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.common.ConfirmForm");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.review.ParticipantResultSaver");
dojo.require("com.ibm.rdm.web.review.reviewChangeNotifier");
dojo.require("com.ibm.rdm.web.commenting.ReviewCommentForm");
dojo.require("com.ibm.rdm.web.commenting.CommentingClient");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewUpdateMediator");


var ParticipantResultSaver = com.ibm.rdm.web.review.ParticipantResultSaver;
var ReviewCache = com.ibm.rdm.web.review.ReviewCache;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var UpdateContext = com.ibm.rdm.web.common.UpdateContext;
var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewUpdateMediator");
var reviewChangeNotifier = com.ibm.rdm.web.review.reviewChangeNotifier;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewutils = com.ibm.rdm.web.review.reviewutils;


com.ibm.rdm.web.review.reviewUpdateMediator = {
	// summary:
	//    This singleton serves as a mediator to coordinate the work that goes
	//    on when casting votes within an ApprovalEditor or within an
	//    ArtifactInReviewViewer.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_okButtonHandle: null,
	_cancelButtonHandle: null,
	_reviewCommentForm: null,
	
	///////////////////////////////////////////////////////////////////////////
	// API

	castArtifactVotes: function(
			reviewParser, myChangedArtifactInfos, doPromptForComments,
			onSaveComplete, onSaveCancelled, onSaveError) {
		// summary:
		//    Save away new artifact vote casts for the the current user.
		// reviewParser: object
		//    The ReviewParser instance that was originally used to hold
		//    the pre-tweaked myChangedArtifactInfos (i.e. before votes were
		//    "cast" within them by changing their myArtifactStatusType
		//    property field value).
		// myChangedArtifactInfos: Array of artifactInfos
		//    Please note that the artifactInfos found in this array represent
		//    ONLY those artifactInfo's whose vote setting has changed (i.e.
		//    it is NOT intended to represent the complete list of new values
		//    to record [changed vote values as well as original vote values]).
		// doPromptForComments: Boolean
		//    Only prompt for comments when this value is set to true.
		// onSaveComplete: function
		//    A callback that will be invoked when the save has successfully
		//    completed.
		// onSaveCancelled: function
		//    A callback that will be invoked when the user decides to
		//    Cancel the vote cast he/she was about to make.
		// onSaveError: function
		//    A callback that will be invoked when the POST or PUT on the
		//    participant results fails.  For example, perhaps the review
		//    has been deleted (error code 410).  Callers should be aware
		//    that when errors occur, the ErrorSelectionWidget shows the
		//    error BEFORE is callback is invoked.
		if (doPromptForComments) {
			if (this._reviewCommentForm == null) {
				this._createReviewCommentForm(myChangedArtifactInfos);
			} else {
				this._initializeReviewCommentForm(myChangedArtifactInfos);
			}
			var self = this;
			// TODO (LPK) need to handle close button, busy cursor stays around
			this._okButtonHandle = dojo.connect(this._reviewCommentForm._okButtonNode, "onClick", function(event) {
				self._disconnectHandles();
				self._castArtifactVotesAndAddComments(
					reviewParser, myChangedArtifactInfos,
					onSaveComplete, onSaveCancelled, onSaveError,
					self._reviewCommentForm.commentText);
			});
			this._cancelButtonHandle = dojo.connect(this._reviewCommentForm._cancelButtonNode, "onClick", function(event) {
				self._disconnectHandles();
				onSaveCancelled();
			});
			this._reviewCommentForm.show();
		} else {
			this._castArtifactVotesAndAddComments(
				reviewParser, myChangedArtifactInfos,
				onSaveComplete, onSaveCancelled, onSaveError, "");
		}
	},

	markMyReviewStatusAsDone: function(reviewParser, onSaveComplete,
									   onSaveCancelled, onSaveError) {
		// summary:
		//    Mark the current user's reviewing state as "Done" for the
		//    entire review.
		// reviewParser: object
		//    A ReviewParser instance.
		// onSaveComplete: function
		//    A callback that will be invoked when the save has successfully
		//    completed.
		// onSaveComplete: function
		//    A callback that will be invoked when the save has been cancelled
		var self = this;
		// Get confirmation from the user if artifacts are in progress or not started.  Only continue
		// with the save if confirmation received.
		var artifactInfos = reviewParser.getArtifactInfos();
		var abstainedArtifactInfos = new Array();
		for (var i = 0; i < artifactInfos.length; i++) {
			var artifactInfo = artifactInfos[i];
			if (artifactInfo.myArtifactStatusType == reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted ||
				artifactInfo.myArtifactStatusType == reviewTypes.ARTIFACT_STATUS_TYPES.InProgress) {
				abstainedArtifactInfos.push(artifactInfo);
			}
		}
		if (abstainedArtifactInfos.length > 0) {
			var message = null;
			if (abstainedArtifactInfos.length == 1) {
				message = messages.confirmDoneSingleMessage;
			} else {
				message = dojo.string.substitute(messages.confirmDoneMessage, [abstainedArtifactInfos.length]);
			}
			var form = this._createConfirmForm(message);
			dojo.connect(form._okButtonNode, "onClick", function(event) {
				self._markMyReviewStatusAsDone(
					reviewParser, onSaveComplete, onSaveError, abstainedArtifactInfos);
			});
			dojo.connect(form._cancelButtonNode, "onClick", function(event) {
				onSaveCancelled();
			});
			form.show();
		} else {
			this._markMyReviewStatusAsDone(
				reviewParser, onSaveComplete, onSaveError, abstainedArtifactInfos);
		}
	},

	artifactUnderReviewHasBeenOpened: function(reviewParser, artifactInfo) {
		// summary:
		//    If appropriate, change the state of the artifact under review
		//    from "Not Started" to "In Progress".
		// description:
		//    The first time a review participant opens an artifact under
		//    review from a Started review, a state transition is supposed to
		//    occur for the user's artifact result.  If the artifact's current
		//    status is "Not Started" we move it to "In Progress", otherwise
		//    we do nothing.
		if (reviewutils.canVote(reviewParser)
			&& reviewParser.getReviewState() === reviewTypes.REVIEW_STATE_TYPES.Started
			&& artifactInfo.myArtifactStatusType === reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted
			) {
			artifactInfo.myArtifactStatusType = reviewTypes.ARTIFACT_STATUS_TYPES.InProgress;
			var self = this;
			var onParticipantResultSaverSaveHasCompleted = function() {
				self._updateReviewCacheAndSendChangeNotifications(reviewParser, true);
			};
			var participantResultSaver = new ParticipantResultSaver({
				reviewParser: reviewParser
			});
			participantResultSaver.castArtifactVotes(
				[ artifactInfo ],
				onParticipantResultSaverSaveHasCompleted
			);
		} else {
			// Send out change notifications.  We do this because opening a new
			// ArtifactInReviewViewer results in getting the most up to date
			// information about the Review it's bound to (represented by
			// reviewParser) and this information MAY contain updates to the
			// information sent out the last time the notifications were sent
			// out for this Review.  For example, perhaps the Review state has
			// changed from Started to Paused, in which case appropriate UI
			// updates would need to be performed in the viewers bound to this
			// Review.
			reviewChangeNotifier.reviewHasBeenUpdated(reviewParser);
		}
	},

	reviewHasBeenOpened: function(reviewParser) {
		// summary:
		//    If the Review that's just been opened has a more recent tag than
		//    the last one loaded by the ReviewCache, we need to have the
		//    ReviewCache reload it and send out change notifications.
		var revisionLastLoaded =
			ReviewCache.getRevisionLastLoadedForReview(reviewParser.getSelfUri());
		if (revisionLastLoaded && (revisionLastLoaded != reviewParser.getRevision())) {
			this._updateReviewCacheAndSendChangeNotifications(reviewParser);
		}
	},

	handleRefresh: function(reviewParser) {
		var onReviewLoaded = function(response) {
			var reviewParser = response.doc;
			reviewChangeNotifier.reviewHasBeenUpdated(reviewParser);
			return response;
		};
		var deferred = ReviewCache.getReview(reviewParser.getReviewId());
		deferred.addCallback(onReviewLoaded);
	},

	reviewSaveInProgress: function(reviewParser) {
		// summary:
		//    A user's participant results is about to be saved to the server.
		// description:
		//    Make sure that registered observers are told that the review is
		//    about to be saved.
		// reviewParser:
		//    A ReviewParser object that contains the last loaded review
		//    information as well as the vote cast changes that the current
		//    user is attempting to save.
		reviewChangeNotifier.reviewSaveIsInProgress(reviewParser);
	},

	reviewSaveHasCompleted: function(reviewParser, participantResultUri, revision) {
		// summary:
		//    This function is called whenever a "save participant results"
		//    request has succeeded.
		// description:
		//    An attempt at saving the current user's participant results has
		//    just succeeded.  Fold the information found in the
		//    'participantResultUri' and 'revision' arguments into the
		//    'reviewParser' and forward this updated 'reviewParser' to the
		//    appropriate function in the reviewChangeNotifier (so that
		//    registered observers are told that the review has just been
		//    successfully saved).
		// reviewParser:
		//    A ReviewParser object that contains the last loaded review as
		//    well as vote cast changes that the current user has just
		//    succeeded in saving.
		// participantResultUri:
		//    The participant results URI that was just successfully saved.
		// revision:
		//    The new revision associated with the newly saved participant
		//    results URI.
		reviewParser.setParticipantResultUri(participantResultUri);
		reviewParser.setParticipantResultRevision(revision);
		reviewChangeNotifier.reviewSaveHasCompleted(reviewParser);
	},

	reviewSaveHasFailed: function(reviewParser, participantResultUri, errorStatus) {
		// summary:
		//    This function is called whenever a "save participant results"
		//    request has returned with an error code.
		// description:
		//    Make sure that registered observers are told that the review save
		//    request failed.
		// reviewParser:
		//    A ReviewParser object that contains the last loaded review
		//    information as well as the vote cast changes that the current
		//    user failed in attempting to save.
		reviewChangeNotifier.reviewSaveHasFailed(reviewParser, errorStatus);
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_disconnectHandles: function() {
		dojo.disconnect(this._cancelButtonHandle);
		dojo.disconnect(this._okButtonHandle);
		this._cancelButtonHandle = null;
		this._okButtonHandle = null;
	},

	_updateReviewCacheAndSendChangeNotifications: function(reviewParser, /*Boolean?*/doExpectVotesFromMe) {
		// Note: We need to make sure to change the UpdateContext so that
		// the ReviewCache.getReview function will know that it needs to
		// bother querying the server for the new version of the Review
		// artifact.
		UpdateContext.cachedArtifactHasBeenSaved();
		var onReviewLoaded = function(response) {
			var reviewParser = response.doc;
			reviewChangeNotifier.reviewHasBeenUpdated(reviewParser);
			return response;
		};
		var deferred = ReviewCache.getReview(reviewParser.getReviewId(), doExpectVotesFromMe);
		deferred.addCallback(onReviewLoaded);
	},

	_castArtifactVotesAndAddComments: function(reviewParser, myChangedArtifactInfos,
											   onSaveComplete, onSaveCancelled,
											   onSaveError, commentText) {
		var self = this;
		var participantResultSaver = new ParticipantResultSaver({
			reviewParser: reviewParser
		});
		var onParticipantResultSaverSaveHasCompleted = function() {
			// Note: We need to make sure to change the UpdateContext so that
			// the ReviewCache.getReview function will know that it needs
			// bother querying the server for the new version of the Review
			// artifact.
			UpdateContext.cachedArtifactHasBeenSaved();
			var onReviewLoaded = function(response) {
				var reviewParser = response.doc;
				onSaveComplete();
				reviewChangeNotifier.reviewHasBeenUpdated(reviewParser);
 				self._createComments(
					reviewParser, myChangedArtifactInfos, commentText);
				return response;
			};
			var deferred = ReviewCache.getReview(
				reviewParser.getReviewId(),
				true			// doExpectVotesFromMe
			);
			deferred.addCallback(onReviewLoaded);
		};
		participantResultSaver.castArtifactVotes(
			myChangedArtifactInfos,
			onParticipantResultSaverSaveHasCompleted,
			onSaveError
		);
	},
	
	_createComments: function(reviewParser, myChangedArtifactInfos, commentText) {
		var self = this;
		var commentingWidgets = dijit.registry.byClass("com.ibm.rdm.web.commenting.CommentingWidget");
		var commentAddedCount = 0;
		dojo.forEach(myChangedArtifactInfos, function(artifactInfo) {
			var subject = null;
			var resourceURI = artifactInfo.resourceURI;
			var priority = com.ibm.rdm.web.commenting.PRIORITY_DEFAULT;
			switch (artifactInfo.myArtifactStatusType) {
				case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Approved:				
					subject = dojo.string.substitute(messages.approvedSubject, [reviewParser.getReviewName()]);
					break;
				case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved:
					subject = dojo.string.substitute(messages.disapprovedSubject, [reviewParser.getReviewName()]);
					var priority = com.ibm.rdm.web.commenting.PRIORITY_HIGH;
					break;
				case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Abstained:
					subject = dojo.string.substitute(messages.abstainedSubject, [reviewParser.getReviewName()]);
					break;
				case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed:
					subject = dojo.string.substitute(messages.reviewedSubject, [reviewParser.getReviewName()]);
					break;
				case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.InProgress:
					subject = dojo.string.substitute(messages.abstainedSubject, [reviewParser.getReviewName()]);
					break;
				case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted:
					subject = dojo.string.substitute(messages.abstainedSubject, [reviewParser.getReviewName()]);
					break;
			};
			if (subject == null) { 
				var msg = dojo.string.substitute(messages.invalidArtifactStatus, [artifactInfo.myArtifactStatusType]);
				var details = "function reviewUpdateMediator._createComments"; // NON-NLS
				com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(
						msg, details
				);
			} else {
				var newComment = new com.ibm.rdm.web.commenting.Comment({
					locationId: "", // artifact level comment 
					reviewId: reviewParser.getReviewId(),
					text: commentText,
					title: subject,
					priority: priority, 
					resourceUri: resourceURI,
					parentComment: "",
					rootComment: "",
					directedToUsers: new Array(),
	               	isResolved: false,
	               	isRoot: true
				});
				var onCommentAdded = function(response) {
					commentAddedCount++;
					if (commentAddedCount == myChangedArtifactInfos.length) {
						// All the comments have been created, update the review's artifact comments widget to reflect
						// the new comments.
						var reviewCommentsWidget = null;
						var reviewCommentsWidgets = dijit.registry.byClass("com.ibm.rdm.web.review.ReviewArtifactsCommentWidget");
						reviewCommentsWidgets.forEach(function(w){
		    				if (w.resourceURI == reviewParser.getSelfUri()) {
								reviewCommentsWidget = w;
		    				}
						});
						if (reviewCommentsWidget) {
							reviewCommentsWidget.refreshComments();
						}
					}
					return response;
				};
				var commentingWidget = null;
				// see if the commenting widget exists for the resource
				commentingWidgets.forEach(function(w){
		    		if ((w.resourceURI == resourceURI) && w.isCommentsCreated()) {
						commentingWidget = w;
		    		}
				});
				var commentDeferred = null;
				if (commentingWidget == null) {
					commentDeferred = com.ibm.rdm.web.commenting.CommentingClient.saveNewComment(newComment, reviewParser.getProjectUri());
				} else {
					// ask the controller to save the comment so the sidebar gets updated properly
					commentDeferred = commentingWidget._controller.saveNewComment(newComment);
				}
				commentDeferred.addCallback(onCommentAdded);
				commentDeferred.addErrback(this, function(error) {
					var msg = messages.commentCreateFailed;
					com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(
						msg, error
					);
				});
			 };
		});
	},

	_createReviewCommentForm: function(artifactInfos) {
		// Reuse the existing form
		var artifactInfo = artifactInfos[0];
		var form = new com.ibm.rdm.web.commenting.ReviewCommentForm(
		   {
				id: "com.ibm.rdm.web.commenting.ReviewCommentForm",
				artifactStatus: artifactInfo.myArtifactStatusType,
				isMultipleArtifacts: (artifactInfos.length > 1)
		   }
		);   
		var dialogWidget = new dijit.Dialog(
			{
				id: "com.ibm.rdm.web.commenting.ReviewCommentDialog",
				duration: 1 // popup immediately
			},
			form.domNode
		);
		form.setDialogWidget(dialogWidget);
		form.formData = {};
		this._reviewCommentForm = form;
	},
	
	_initializeReviewCommentForm: function(artifactInfos) {
		// Reuse the existing form
		var artifactInfo = artifactInfos[0];
		this._reviewCommentForm.artifactStatus = artifactInfo.myArtifactStatusType;
		this._reviewCommentForm.isMultipleArtifacts = (artifactInfos.length > 1);
		this._reviewCommentForm.initialize();
		this._reviewCommentForm.formData = {};
	},
	
	_createConfirmForm: function(message) {
		var form = new com.ibm.rdm.web.common.ConfirmForm(
		   {
				id: "com.ibm.rdm.web.review.ConfirmForm",
				dialogTitle: messages.confirmDone,
				confirmMessage: message
		   }
		);   
		var dialogWidget = new dijit.Dialog(
			{
				id: "com.ibm.rdm.web.review.ConfirmDialog",
				duration: 1 // popup immediately
			},
			form.domNode
		);
		form.setDialogWidget(dialogWidget);
		return form;
	},
	
	_markMyReviewStatusAsDone: function(reviewParser, onSaveComplete,
										onSaveError, abstainedArtifactInfos) {
		var self = this;
		var participantResultSaver = new ParticipantResultSaver({
			reviewParser: reviewParser
		});
		var onParticipantResultSaverSaveHasCompleted = function() {
			// Note: We need to make sure to change the UpdateContext so that
			// the ReviewCache.getReview function will know that it needs
			// bother querying the server for the new version of the Review
			// artifact.
			UpdateContext.cachedArtifactHasBeenSaved();
 			self._createComments(reviewParser, abstainedArtifactInfos, "");
			var onReviewLoaded = function(response) {
				var reviewParser = response.doc;
				// We just obtained a new instance of a ReviewParser which
				// accurately reflects the newly saved artifact votes.
				// Now we need to inform any artifacts that might need to
				// update UI elements to reflect these changes.
				onSaveComplete();
				reviewChangeNotifier.reviewHasBeenUpdated(reviewParser);
				return response;
			};
			var deferred = ReviewCache.getReview(
				reviewParser.getReviewId(),
				true			// doExpectVotesFromMe
			);
			deferred.addCallback(onReviewLoaded);
		};
		participantResultSaver.markMyReviewStatusAsDone(
			onParticipantResultSaverSaveHasCompleted, onSaveError);
	}
};

})();
