/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.sidebar.SidebarLayout");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.requireLocalization("com.ibm.rdm.web.sidebar", "SidebarLayout");

/*
   Copy from JAF (pre 0.6 release). Purpose: Starting point for our control to
   enable us to use a similar expandable/collapsible content area but on the
   right hand side of our UI (rather than the left hand side as JAF does).
*/

/**
 * Note: This class makes use of code found in JAF's SideNavigationLayout.js to
 * implement the draggable/expandable/collapsable vertical divider (all functions
 * in this file that are based on functions found in SideNavigationLayout.js have
 * a comment mentioning this fact).
 *
 * In addition to the slider concept/implementation borrowed from JAF's
 * SideNavigationLayout this class implements "sidebar sections" that are
 * independently expandable/collapsable.  A particular section may also be
 * maximized or restored back to its original collapsed or expanded state.
 *
 * Care is taken to avoid needlessly "starting" a particular section until the
 * first time it's expanded or maximized.  Therefore you may choose to add a
 * section that's initially collapsed to improve startup time.  Only AFTER the
 * first time the user expands or maximizes the section will the 'startup'
 * function of the underlying widget be called (and therefore your widget should
 * defer making server calls until its 'startup' function has been called).
 *
 * All widgets added as sections via 'addSidebarSection' should implement the
 * following functions:
 *     containerDimensionsHaveChanged()
 *         [Called whenever the sidebar's dimensions change.]
 *     handleRefresh()
 *         [Called whenever SidebarLayout's 'handleRefresh' function has been
 *          called.]
 *
 * In addition to the sidebar section widgets, there's another widget managed by
 * this class that represents the "main content area" found to the left of the
 * draggable/expandable/collapsable vertical divider.  This widget should
 * implement the following functions:
 *     setDimensions(width, height)
 *         [Called whenever the dimensions of the "main content area" need
 *          to change (due to either either the browser window resizing or the
 *          vertical divider dragging, expanding or collapsing).]
 *     handleRefresh()
 *         [Called whenever SidebarLayout's 'handleRefresh' function has been
 *          called.]
 *
 * Example usage:
 *     var mainContentWidget = new MyMainContentWidget();
 *     var section1Widget = new MySectionWidget({});
 *     var section2Widget = new MySectionWidget({});
 *     var sidebarLayout = new com.ibm.rdm.web.sidebar.SidebarLayout({
 *         contentNode: mainContentWidget.domNode});
 *     sidebarLayout.addSidebarSection("Section One", section1Widget.domNode, true);
 *     sidebarLayout.addSidebarSection("Section Two", section2Widget.domNode, false);
 *     sidebarLayout.startup();
 */
dojo.declare("com.ibm.rdm.web.sidebar.SidebarLayout", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "sidebar/templates/SidebarLayout.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_contentArea: null,
	_showHideControl: null,
	_showHideButton: null,
	_sidebarArea: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	// contentNode: A DOM node or a Widget
	//    The content node that is to appear to the left of the sidebar as the
	//    "main content".
	contentNode: null,
	// bottomOffset: Integer
	//    The page bottom offset, in pixels, at which the bottom for the sidebar
	//    (and its content pane) should appear.  This affordance could be used,
	//    for example, to allow for use of this SidebarLayout in the context of
	//    a page in which a footer has been specified whose CSS includes
	//    something like "margin-bottom: -30px;".  In this example case, not
	//    specifying 'bottomOffset' would result in a SidebarLayout whose bottom
	//    appears in alignment with the scroll viewport bottom, but this would
	//    have the unfortunate side effect of pushing the footer down so that you
	//    would have to scroll 30 pixels down to see the footer in its entirety.
	bottomOffset: 0,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_dragHandle: null,
	_mouseOutHandle: null,
	_mouseUpHandle: null,
	_sidebarAreaWidth: null,
	_currentSidebarState: null,
	_sections: null,
	_sidebarWidthBeforeToggle: null,
	_sidebarAreaAbsolutePosY: null,
	_lastScrollY: null,
	_sidebarSectionBeingAnimated: null,
	_adjustSectionHeightsUsingAnimation: false,
	_contentWidget: null,
	_offsetTop: null,
	_startupHasBeenHandled: false,
	_messages: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._startupHasBeenHandled = false;
		this._initDimensions();
		this._sections = [];
		if (this.contentNode) {
			this.setContent(this.contentNode);
		}
		// Hook up events for the _showHideControl.
		this.connect(this._showHideControl, "ondblclick", this._toggleSidebar);
		this.connect(this._showHideControl, "onmouseover", this._highlightShowHideControl);
		this.connect(this._showHideControl, "onmouseout", this._unhighlightShowHideControl);
		this.connect(this._showHideControl, "onmousedown", this._startDragging);
		// Hook up events for the _showHideButton.
		this.connect(this._showHideButton, "onclick", this._toggleSidebar);
		this.connect(this._showHideButton, "onmouseover", this._highlightShowHideButton);
		this.connect(this._showHideButton, "onmouseout", this._unhighlightShowHideButton);
		this.connect(this._showHideButton, "onmousedown", this._cancelEvent);
		this.connect(this._showHideButton, "onmouseup", this._cancelEvent);
		this.connect(this._showHideButton, "onkeypress", this._onKeyPress);
	},
	
	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.sidebar", "SidebarLayout");
	},

	setContent: function(contentNode) {
		// summary:
		//    In some cases, you may to prefer deferring setting the sidebar
		//    layout's content node until AFTER construction.  This function
		//    is provided to handle such cases.
		this._contentWidget = (contentNode.nodeType == 1 && dijit.byNode(contentNode));
		this._contentArea.innerHTML = "";
		this._contentArea.appendChild(contentNode);
		this.contentNode = contentNode;
	},

	addSidebarSection: function(sectionTitle, sectionContentNode, showAsExpanded) {
		var section = new com.ibm.rdm.web.sidebar._SidebarSection({
			sectionTitle: sectionTitle,
			contentNode: sectionContentNode,
			showAsExpanded: showAsExpanded,
			sidebarLayout: this});
		this._sidebarArea.appendChild(section.domNode);
		this._sections.push(section);
		return section;
	},

	startup: function() {
		this._offsetTop = dojo._abs(this.domNode).y;
		if (this._contentWidget) {
			this._contentWidget.startup();
		}
		for (var i = 0; i < this._sections.length; i++) {
			this._sections[i].startup();
		}
		// Add a div at the top to account for the SIDEBAR_TOP_MARGIN
		if (this._sections.length > 0) {
			var firstSection = this._sections[0];
			/*var div = document.createElement("div");
			div.style.height =
				(SIDEBAR_TOP_MARGIN - firstSection.getBoxModelTopGap()) + "px";
			this._sidebarArea.insertBefore(div, firstSection.domNode);*/
		}
		// Note: We're defering use of animation until _after_ the initial display.
		this._adjustSectionHeightsUsingAnimation = false;
		this.setAllExpandedSectionsToSameHeight(true);
		var scrollPos = dojo._docScroll();
		var absolutePos = dojo._abs(this._sidebarArea);
		this._sidebarAreaAbsolutePosY = absolutePos.y + scrollPos.y;
		this._setWidthForContentAreaIfNeeded();
		// Okay -- now we'll use animation if needed ...
		this._adjustSectionHeightsUsingAnimation = ADJUST_SECTION_HEIGHTS_USING_ANIMATION;
		this._startupHasBeenHandled = true;
		
		// Hook up events for window resizing.
		this._resizeEvent = jazz.util.ResizeNotifier.register({context: this, callback: this.handleWindowResize, force: true});
	},

	destroy: function() {
		this._sidebarSectionBeingAnimated = null;
		jazz.util.ResizeNotifier.unregister(this._resizeEvent);
		for (var i = 0; i < this._sections.length; i++) {
			this._sections[i].destroyRecursive();
			this._sections[i] = null;
		}
		this._sections = [];
		if (this._contentWidget) {
			this._contentWidget.destroyRecursive();
			this._contentWidget = null;
		}
		this.inherited(arguments);
	},

	setAllExpandedSectionsToSameHeight: function(forceNoAnimation/*improves performance!!!*/, sidebarSectionUndergoingHeightChange) {
		if (ADJUST_SECTION_HEIGHTS_USING_ANIMATION && !forceNoAnimation && this._adjustSectionHeightsUsingAnimation) {
			if (!sidebarSectionUndergoingHeightChange) {
				for (var i = 0; i < this._sections.length; i++) {
					var section = this._sections[i];
					if (section.isExpanded()) {
						sidebarSectionUndergoingHeightChange = section;
						break;
					}
				}
			}
			// Note: If *all* sections are collapsed sidebarSectionUndergoingHeightChange
			// will be set to undefined.
			if (sidebarSectionUndergoingHeightChange) {
				this._adjustExpandedSectionHeightsThroughAnimation(sidebarSectionUndergoingHeightChange);
			}
		} else {
			this._adjustExpandedSectionHeights();
		}
	},

	getNumExpandedSections: function() {
		var numExpandedSections = 0;
		for (var i = 0; i < this._sections.length; i++) {
			if (this._sections[i].isExpanded()) {
				numExpandedSections++;
			}
		}
		return numExpandedSections;
	},

	redrawSectionTitleIcons: function() {
		for (var i = 0; i < this._sections.length; i++) {
			this._sections[i].redrawTitleIcons();
		}
	},

	maximizeSection: function(sidebarSectionToMaximize) {
		// TODO Do a better job with animation here
		var isExpanded = sidebarSectionToMaximize.isExpanded();
		for (var i = 0; i < this._sections.length; i++) {
			var sidebarSection = this._sections[i];
			sidebarSection.setExpanded(sidebarSection === sidebarSectionToMaximize);
			if (sidebarSection !== sidebarSectionToMaximize) {
				sidebarSection.setMaximized(false, true);
			}
		}
		this.setAllExpandedSectionsToSameHeight(false, sidebarSectionToMaximize);
	},

	handleRefresh: function() {
		if (this._contentWidget && this._contentWidget.handleRefresh) {
			this._contentWidget.handleRefresh();
		}
		for (var i = 0; i < this._sections.length; i++) {
			this._sections[i].handleRefresh();
		}
	},

	getSidebarSectionWidth: function() {
		return this._sidebarAreaWidth;
	},

	handleWindowResize: function() {
		// Note: If width is set to 0, it's an indication
		// that this widget has been orphaned and therefore we shouldn't
		// perform any resizing ...
		if (this._startupHasBeenHandled && dojo.coords(this.domNode).w > 0) {
			this.setAllExpandedSectionsToSameHeight(true);
			this._setWidthForContentAreaIfNeeded();
		}
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_setWidthForContentAreaIfNeeded: function(sidebarAreaWidth) {
		// Note: sidebarAreaWidth is optional.
		var widget = this._contentWidget;
		if (widget && widget.setDimensions) {
			if (!sidebarAreaWidth) {
				sidebarAreaWidth = parseInt(this._sidebarArea.style.width, 10);
			}
			var clientWidth = dojo.coords(document.body).w - JAF_GUTTERS_WIDTH;
			var widthToUse = Math.max(0, (clientWidth - sidebarAreaWidth - SHOW_HIDE_CTRL_WIDTH));
			var heightToUse = this._getSidebarHeight();
			if (heightToUse <= 0) {
				heightToUse = 1;
			}
			widget.setDimensions(widthToUse, heightToUse);
		}
	},

	_getSidebarHeight: function() {
		// TODO Consider using something other than dojo._abs, as the y value provided
		// is float rather than int.
		var sidebarY = dojo._abs(this._sidebarArea).y;
		var sidebarHeight = document.documentElement.clientHeight - sidebarY - this.bottomOffset;
		return sidebarHeight;
	},

	_adjustExpandedSectionHeights: function() {
		// summary:
		//     Adjusts the height all expanded sections so that they share a
		//     common height.  No animation is used when performing these
		//     section height adjustments.
		var bottomGap = this._sections[0].getBoxModelBottomGap();
		var sidebarHeight = this._getSidebarHeight() - bottomGap;
		var numExpandedSections = 0;
		for (var i = 0; i < this._sections.length; i++) {
			if (this._sections[i].isExpanded()) {
				numExpandedSections++;
			}
		}
		var collapsedSectionHeight = this._sections[0].getCollapsedHeight();
		var totalHeightOfCollapsedSections =
			(this._sections.length - numExpandedSections) * collapsedSectionHeight;
		var heightForEachExpandedSection =
			Math.floor((sidebarHeight - totalHeightOfCollapsedSections) / numExpandedSections);
		for (var i = 0; i < this._sections.length; i++) {
			var section = this._sections[i];
			if (section.isExpanded()) {
				section.setHeight(heightForEachExpandedSection);
			}
		}
	},

	_adjustExpandedSectionHeightsThroughAnimation: function(sidebarSectionUndergoingHeightChange) {
		// summary:
		//     Adjusts the height all expanded sections so that they share a
		//     common height.  Animation is used when adjusting the section
		//     heights.
		// sidebarSectionUndergoingHeightChange:
		//     A sidebarSection whose height is known to be changing.
		this._sidebarSectionBeingAnimated = sidebarSectionUndergoingHeightChange;
		var currentHeight = sidebarSectionUndergoingHeightChange.getHeight();
		var targetHeight = (
			sidebarSectionUndergoingHeightChange.isExpanded() ?
			this._getCommonHeightToUseForExpandedSections() :
			sidebarSectionUndergoingHeightChange.getCollapsedHeight());
		var anim = new dojo._Animation({
			curve: [currentHeight, targetHeight],
			duration: 150
		});
		this.connect(anim, "onAnimate", this._onResizeExpandedSectionsInProgress);
		anim.play();
	},

	_onResizeExpandedSectionsInProgress: function(sidebarSectionHeight) {
		sidebarSectionHeight = Math.floor(sidebarSectionHeight);
		this._sidebarSectionBeingAnimated.setHeight(sidebarSectionHeight);
		this._setOtherExpandedSectionsToSameHeight(this._sidebarSectionBeingAnimated, sidebarSectionHeight);
	},

	_setOtherExpandedSectionsToSameHeight: function(sidebarSectionToExclude, excludedSidebarSectionHeight) {
		var heightForEachExpandedSection = this._getCommonHeightToUseForExpandedSections(
			sidebarSectionToExclude, excludedSidebarSectionHeight);
		for (var i = 0; i < this._sections.length; i++) {
			var section = this._sections[i];
			if (section !== sidebarSectionToExclude && section.isExpanded()) {
				section.setHeight(heightForEachExpandedSection);
			}
		}
	},

	_getCommonHeightToUseForExpandedSections: function(sidebarSectionToExclude, excludedSidebarSectionHeight) {
		// Note: the two formal args are optional.
		var collapsedSectionHeight = this._sections[0].getCollapsedHeight();
		var bottomGap = this._sections[0].getBoxModelBottomGap();
		var sidebarHeight = this._getSidebarHeight() - bottomGap;
		if (excludedSidebarSectionHeight) {
			sidebarHeight -= excludedSidebarSectionHeight;
		}
		var numExpandedSections = 0;
		var numCollapsedSections = 0;
		for (var i = 0; i < this._sections.length; i++) {
			var section = this._sections[i];
			var doExclude = (section === sidebarSectionToExclude);
			if (section !== sidebarSectionToExclude) {
				if (section.isExpanded()) {
					numExpandedSections++;
				} else {
					numCollapsedSections++;
				}
			}
		}
		var totalHeightOfCollapsedSections =
			numCollapsedSections * collapsedSectionHeight;
		var heightForEachExpandedSection =
			Math.floor((sidebarHeight - totalHeightOfCollapsedSections) / numExpandedSections);
		return heightForEachExpandedSection;
	},

	_handleScroll: function(e) {
		var scrollPos = dojo._docScroll();
		var scrollbarHeightNeedsToChange = (
			(this._lastScrollY > this._sidebarAreaAbsolutePosY
			 && scrollPos.y <= this._sidebarAreaAbsolutePosY)
			|| (this._lastScrollY <= this._sidebarAreaAbsolutePosY
				&& scrollPos.y > this._sidebarAreaAbsolutePosY)
			|| (this._lastScrollY <= this._sidebarAreaAbsolutePosY
				&& scrollPos.y <= this._sidebarAreaAbsolutePosY));
		var absolutePos = dojo._abs(this._sidebarArea);
		if (scrollPos.y <= this._sidebarAreaAbsolutePosY) {
			this._sidebarArea.style.top = "";
			this._sidebarArea.style.left = "0px";
		} else {
			// We need to make sure that the top of the sidebar is visible.
			// Note: We're not using a position setting of "fixed" as IE<7
			// doesn't support a "fixed" position.
			this._sidebarArea.style.top = (scrollPos.y - this._sidebarAreaAbsolutePosY)+ "px";
			this._sidebarArea.style.left = "0px";
		}
		if (scrollbarHeightNeedsToChange) {
			this.setAllExpandedSectionsToSameHeight(true);
		}
		this._lastScrollY = scrollPos.y;
	},
	
	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_initDimensions: function() {
		// summary:
		//    Set the initial state of the elements.
		if (dojo.isIE == 6) {
			this._showHideButton.style.visibility = "hidden";
			this._showHideControl.style.cursor = "normal";
		}
		this._setSidebarCurrentState("open");
		this._sidebarArea.style.width = SIDEBAR_DEFAULT_WIDTH + "px";
		this._sidebarAreaWidth = SIDEBAR_DEFAULT_WIDTH;
		dojo.addClass(this._showHideButton, "sidebarShowHideButtonExpanded");
		dojo.removeClass(this._showHideButton, "sidebarShowHideButtonCollapsed");
		this._lastScrollY = 0;
	},
	
	// Note: this function is based on a function named "_toggleLeftNav"
	// JAF's SideNavigationLayout.js ... however, it's been changed quite a bit.
	_toggleSidebar: function(e) {
		// disable left nav resizing on IE6
		if (!(dojo.isIE && dojo.isIE <= 6)) {
			// description:
			//        If the left-nav is open, close it. If the left-nav is closed,
			//        open it.
			if (dojo.isIE >= 7) {
				this._showHideButton.style.visibility = "hidden";
			}
			// animate by changing the width
			var args = {
				node: this._sidebarArea,
				duration: 250,
				properties: { width: { unit: "px" } }
			};
			if (this._currentSidebarState === "open") {
				// hide
				this._sidebarWidthBeforeToggle = parseInt(this._sidebarArea.style.width, 10);
				args.properties.width.start = this._sidebarWidthBeforeToggle;
				args.properties.width.end = 0;
				var anim = dojo.animateProperty(args);
				this.connect(anim, "onEnd", this._onHideAnimationEnd);
			}
			else {
				// show
				args.properties.width.start = 0;
				args.properties.width.end = this._sidebarWidthBeforeToggle;
				var anim = dojo.animateProperty(args);
				this.connect(anim, "onEnd", this._onShowAnimationEnd);
			}
			anim.play();
		}
	},
	
	_onKeyPress: function(event) {
		if(event.charCode == dojo.keys.SPACE) { 
			dojo.stopEvent(event);
			this._toggleSidebar();
		}
	},
	
	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_onHideAnimationEnd: function() {
		this._setSidebarCurrentState("close");
		dojo.addClass(this._showHideButton, "sidebarShowHideButtonCollapsed");
		dojo.removeClass(this._showHideButton, "sidebarShowHideButtonExpanded");
		if(dojo.isIE >= 7){
			this._showHideButton.style.visibility = "visible";
		}
		this._setWidthForContentAreaIfNeeded();
	},
	
	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_onShowAnimationEnd: function() {
		this._setSidebarCurrentState("open");
		dojo.addClass(this._showHideButton, "sidebarShowHideButtonExpanded");
		dojo.removeClass(this._showHideButton, "sidebarShowHideButtonCollapsed");
		if(dojo.isIE >= 7){
			this._showHideButton.style.visibility = "visible";
		}
		this._setWidthForContentAreaIfNeeded();
	},

	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_highlightShowHideControl: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			dojo.addClass(this._showHideControl, "rdm-sidebar-layout-showHideControl-hover");
			dojo.removeClass(this._showHideControl, "rdm-sidebar-layout-showHideControl");
		}
	},
	
	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_unhighlightShowHideControl: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			dojo.addClass(this._showHideControl, "rdm-sidebar-layout-showHideControl");
			dojo.removeClass(this._showHideControl, "rdm-sidebar-layout-showHideControl-hover");
		}
	},

	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_highlightShowHideButton: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			dojo.addClass(this._showHideButton, "rdm-sidebar-layout-showHideButton-hover");
			dojo.removeClass(this._showHideButton, "rdm-sidebar-layout-showHideButton");
			dojo.addClass(this._showHideControl, "rdm-sidebar-layout-showHideControl");
			dojo.removeClass(this._showHideControl, "rdm-sidebar-layout-showHideControl-hover");
		}
	},
	
	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_unhighlightShowHideButton: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			dojo.addClass(this._showHideButton, "rdm-sidebar-layout-showHideButton");
			dojo.removeClass(this._showHideButton, "rdm-sidebar-layout-showHideButton-hover");
			dojo.addClass(this._showHideControl, "rdm-sidebar-layout-showHideControl-hover");
			dojo.removeClass(this._showHideControl, "rdm-sidebar-layout-showHideControl");
		}
	},
	
	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_startDragging: function(e) {
		// disable left nav resizing on IE6
		if (!(dojo.isIE && dojo.isIE <= 6)) {
			dojo.addClass(document.body, "rdm-sidebar-layout-dragCursor");
			this.disconnect(this._dragHandle);
			this.disconnect(this._mouseOutHandle);
			this.disconnect(this._mouseUpHandle);
			this._dragHandle = this.connect(document.body, "onmousemove", "_drag");
			this._mouseOutHandle = this.connect(document.body, "onmouseout", "_drag");
			this._mouseUpHandle = this.connect(document.body, "onmouseup", "_stopDragging");
			this._isDragging = true;
		}
	},

	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_stopDragging: function(e) {
		dojo.removeClass(document.body, "rdm-sidebar-layout-dragCursor");
		this.disconnect(this._dragHandle);
		this.disconnect(this._mouseOutHandle);
		this.disconnect(this._mouseUpHandle);
		delete this._dragHandle;
		delete this._mouseOutHandle;
		delete this._mouseUpHandle;
		var widthpx = this._sidebarArea.style.width;
		widthpx = widthpx.substring(0, widthpx.length - 2);
		if (widthpx >= SIDEBAR_MIN_WIDTH) {
			this._sidebarAreaWidth = widthpx;
		}
		this._isDragging = false;
	},
	
	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_drag: function(e) {
		// prevent text selection
		e.preventDefault();
		e.cancelBubble = true;
		dojo.stopEvent(e);
		
		// check if mouse button was released
		var x = e.pageX - (SHOW_HIDE_CTRL_WIDTH/2);
		var pageWidth = dojo.coords(document.body).w;
		var newSidebarWidth = pageWidth - x;
		var sidebarLayoutWidth = this._sidebarArea.parentNode.clientWidth;
		var widthToUse;
		if (newSidebarWidth < SIDEBAR_MIN_WIDTH) {
			// snap closed
			widthToUse = 0;
			this._setSidebarCurrentState("close");
			dojo.addClass(this._showHideButton, "sidebarShowHideButtonCollapsed");
			dojo.removeClass(this._showHideButton, "sidebarShowHideButtonExpanded");
		}
		else if (newSidebarWidth > (sidebarLayoutWidth - CONTENT_AREA_MIN_WIDTH)) {
			// prevent further expansion
			widthToUse = (sidebarLayoutWidth - CONTENT_AREA_MIN_WIDTH);
			this._setSidebarCurrentState("open");
			dojo.addClass(this._showHideButton, "sidebarShowHideButtonExpanded");
			dojo.removeClass(this._showHideButton, "sidebarShowHideButtonCollapsed");
		}
		else {
			// slide with mouse
			widthToUse = newSidebarWidth;
			this._setSidebarCurrentState("open");
			if (dojo.isIE == 7) {
				// IE7 bug: doesn't redraw the arrow properly while dragging
				dojo.removeClass(this._showHideButton, "sidebarShowHideButtonExpanded");
			}
			dojo.addClass(this._showHideButton, "sidebarShowHideButtonExpanded");
			dojo.removeClass(this._showHideButton, "sidebarShowHideButtonCollapsed");
		}
		this._showHideControl.style.marginRight = "0px";
		this._sidebarArea.style.width = widthToUse + "px";
		for (var i = 0; i < this._sections.length; i++) {
			this._sections[i].setWidth(widthToUse);
		}
		this._setWidthForContentAreaIfNeeded(widthToUse);
	},
	
	_setSidebarCurrentState: function(state) {
		this._currentSidebarState = state;
		// accessibility
		if (this._currentSidebarState == "open") {
			this._showHideButton.setAttribute("title", this._messages["hideTooltip"]);
		} else {
			this._showHideButton.setAttribute("title", this._messages["showTooltip"]);
		}
	},
	
	// Note: this function is based on a function by the same name found in
	// JAF's SideNavigationLayout.js
	_cancelEvent: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
	}
});

var ADJUST_SECTION_HEIGHTS_USING_ANIMATION = !(dojo.isIE == 6);
var CONTENT_AREA_MIN_WIDTH = 100;
var SIDEBAR_MIN_WIDTH      = 30;
var SIDEBAR_DEFAULT_WIDTH  = 280;
var SHOW_HIDE_CTRL_WIDTH   = 8;
var SIDEBAR_TOP_MARGIN     = 7;
var SIDEBAR_BOTTOM_MARGIN  = 3;
var JAF_GUTTERS_WIDTH = 24;
var SIDEBAR_SECTION_COLLAPSED_HEIGHT = 19;

dojo.declare("com.ibm.rdm.web.sidebar._SidebarSection", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "sidebar/templates/SidebarSection.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_sectionHeader: null,
	_maximizeImage: null,
	_sectionContent: null,
	_messages: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	sectionTitle: null,
	contentNode: null,
	showAsExpanded: false,
	sidebarLayout: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_contentWidget: null,
	_isSectionExpanded: null,
	_isSectionMaximized: null,
	_sectionBoxModelInfo: {
		"margin-top": null,
		"margin-bottom": null,
		"border-top-width": null,
		"border-bottom-width": null
	},
	_headerBoxModelInfo: {
		"margin-top": null,
		"margin-bottom": null,
		"border-top-width": null,
		"border-bottom-width": null,
		"height": null
	},

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._isSectionExpanded = this.showAsExpanded;
		this._isSectionMaximized = false;
		this._setMaximizedStateIcon();
		this._maximizeStateImage.setAttribute("title", this._messages["maximizeTooltip"]);
		this._maximizeStateImage.setAttribute("alt", this._messages["maximizeAlt"]);
		this._setExpandedStateIcon();
		this.connect(this._sectionHeader, "onclick", this._onSectionHeaderClick);
		this.connect(this._sectionHeader, "onmouseover", this._onSectionHeaderMouseOver);
		this.connect(this._sectionHeader, "onmouseout", this._onSectionHeaderMouseOut);
		this.connect(this._maximizeStateImage, "onclick", this._showAsMaximized);
		this.connect(this._sectionHeader, "ondblclick", this._showAsMaximized);
		this.connect(this._sectionHeader, "onkeypress", this._onKeyPressSectionHeader); 
		this.connect(this._maximizeStateImage, "onkeypress", this._onKeyPressMaximize); 
		this.connect(this._expandStateImage, "onkeypress", this._onKeyPressExpand); 
		
		if (dojo.isIE) {
			dojo.addClass(this._sectionHeader, "section-header-IE");
		}
		
		//let every widget have a handle on the sidebar section
		this._contentWidget = (this.contentNode.nodeType == 1 && dijit.byNode(this.contentNode));
		if (this._contentWidget)
			this._contentWidget._sidebarSection = this;
	},

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.sidebar", "SidebarLayout");
	},

	startup: function() {
		this._initBoxModelAttributes();
		if (this._isSectionExpanded) {
			this._addSectionContent();
		} else {
			this._sectionContent.style.display = "none";
		}
		this._adjustTitleArea();
	},

	destroy: function() {
		if (this._contentWidget) {
			this._contentWidget.destroyRecursive();
			this._contentWidget = null;
		}
		this.inherited(arguments);
	},
	
	setSectionTitleInfo: function(message) {
		this.clearSectionTitleInfo();
		this._sectionTitleSpan.appendChild(document.createTextNode(message));
	},
	
	clearSectionTitleInfo: function() {
		this._sectionTitleSpan.innerHTML = "";
	},

	redrawTitleIcons: function() {
		this._adjustTitleArea();
	},

	setHeight: function(height) {
		var contentHeightToUse = (
			height
			- this._sectionHeader.clientHeight
			- this._sectionBoxModelInfo["margin-top"]
			- this._sectionBoxModelInfo["border-top-width"]
			- this._headerBoxModelInfo["margin-top"]
			- this._headerBoxModelInfo["margin-bottom"]
			- this._headerBoxModelInfo["border-top-width"]
			- this._headerBoxModelInfo["border-bottom-width"]
		);
		if (contentHeightToUse < 0) {
			contentHeightToUse = 0;
		}
		this._sectionContent.style.height = contentHeightToUse + "px";
		var widget = this._contentWidget;
		if (widget && widget.containerDimensionsHaveChanged && widget.domNode.parentNode) {
			widget.containerDimensionsHaveChanged();
		}
		if (!widget) {
			this._sectionContent.style.overflow = "auto";
		}
	},

	setWidth: function(height) {
		var widget = this._contentWidget;
		if (widget && widget.containerDimensionsHaveChanged && widget.domNode.parentNode) {
			widget.containerDimensionsHaveChanged();
		}
	},

	getHeight: function() {
		var height = (
			this.domNode.clientHeight
			+ this._sectionBoxModelInfo["margin-top"]
			+ this._sectionBoxModelInfo["border-top-width"]
		);
		return height;
	},

	getCollapsedHeight: function() {
		// Note: We're making use of:
		//     this._headerBoxModelInfo["height"]
		// rather than:
		//     this._sectionHeader.clientHeight
		// as this function may be invoked BEFORE the header icons have been
		// loaded (meaning the clientHeight would not yet be accounting for the
		// height of these icons).
// Note: it seems that we need to account for some additional changes to the CSS
// in order to properly interrogate for height to use ... instead we'll use
// the approach of a SIDEBAR_SECTION_COLLAPSED_HEIGHT global for now ...
// 		var height = (
// 			this._headerBoxModelInfo["height"]
// 			+ this._sectionBoxModelInfo["margin-top"]
// 			+ this._sectionBoxModelInfo["border-top-width"]
// 		);
// 		return height;
		return SIDEBAR_SECTION_COLLAPSED_HEIGHT;
	},

	isExpanded: function() {
		return this._isSectionExpanded;
	},

	setExpanded: function(doShowAsExpanded) {
		if (this._isSectionExpanded !== doShowAsExpanded) {
			this._toggleExpansionState();
			this._sectionContent.style.display = (this._isSectionExpanded ? "" : "none");
		}
		this._adjustTitleArea();
	},
	
	expandSection: function() {
		if (!this._isSectionExpanded) {
			this._toggleExpansionState();
			if (this.isExpanded()) {
				this.setHeight(this.getCollapsedHeight());
			}
			this.sidebarLayout.setAllExpandedSectionsToSameHeight(false, this);
			this._sectionContent.style.display = (this._isSectionExpanded ? "" : "none");
			this._adjustTitleArea();
		}
	},

	setMaximized: function(doShowAsMaximized, /*Optional*/doSuppressInformingSidebarLayout) {
		if (this._isSectionMaximized != doShowAsMaximized) {
			this._showAsMaximized(null, doSuppressInformingSidebarLayout);
		}
		this._adjustTitleArea();
	},

	getBoxModelTopGap: function() {
		return (
			this._sectionBoxModelInfo["margin-top"]
			+ this._sectionBoxModelInfo["border-top-width"]);
	},

	getBoxModelBottomGap: function() {
		return (
			this._sectionBoxModelInfo["margin-bottom"]
			+ this._sectionBoxModelInfo["border-bottom-width"]);
	},

	handleRefresh: function() {
		// It _may_ be the case that this section has never been expanded,
		// in which case we take care to avoid asking it to refresh.
		if (this._sectionContent.firstChild) {
			var widget = this._contentWidget;
			if (widget && widget.handleRefresh) {
				widget.handleRefresh();
			}
		}
	},

	_adjustTitleArea: function() {
		if (this._isSectionExpanded) {
			//dojo.removeClass(this._sectionHeader, "section-header-collapsed");
			dojo.addClass(this._sectionHeader, "section-header-expanded");
		} else {
			//dojo.removeClass(this._sectionHeader, "section-header-expanded");
			dojo.addClass(this._sectionHeader, "section-header-collapsed");
		}
		this._setMaximizedStateIcon();
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_initBoxModelAttributes: function() {
		// e.g. convertToCamelCase("margin-top") ==> "marginTop"
		var convertToCamelCase = function (cssProperty){
			for (
				var exp = /-([a-z])/;
				exp.test(cssProperty);
				cssProperty = cssProperty.replace(exp, RegExp.$1.toUpperCase())
			);
			return cssProperty;
		};
		var populateCssPropList = function(node, propList) {
			if (dojo.isIE) {
				for (var prop in propList) {
					domProp = convertToCamelCase(prop);
					var val = parseInt(node.currentStyle[domProp], 10);
					propList[prop] = (isNaN(val) ? 0 : val);
				}
			} else {
				var css = document.defaultView.getComputedStyle(node, "");
				for (var prop in propList) {
					var val = parseInt(css.getPropertyValue(prop), 10);
					propList[prop] = (isNaN(val) ? 0 : val);
				}
			}
		};
		populateCssPropList(
			this.domNode, this._sectionBoxModelInfo);
		populateCssPropList(
			this._sectionHeader, this._headerBoxModelInfo);
	},

	_showAsMaximized: function(/*Optional*/event, /*Optional*/doSuppressInformingSidebarLayout) {
		if (event) {
			dojo.stopEvent(event);
		}
		if (! doSuppressInformingSidebarLayout) {
			if (! this._isSectionMaximized) {
				// maximize
				this.sidebarLayout.maximizeSection(this);
			}
		}
		this._setMaximizedStateIcon();
	},

	_setMaximizedStateIcon: function() {
		this._isSectionMaximized =
			this._isSectionExpanded && this.sidebarLayout.getNumExpandedSections() === 1;
		this._maximizeStateImage.setAttribute("src", this._getMaximizeStateImageSrc(this._isSectionMaximized));
		// TODO Ask UI folks for appropriate icons.  For time being I'm using two I
		// found in our icons library (one at 16x16 and the other at 11x11).
		this._maximizeStateImage.className = (this._isSectionMaximized ? "restore-image" : "maximize-image");
	},

	_setExpandedStateIcon: function() {
		this._expandStateImage.setAttribute("src", this._getExpandStateImageSrc(this._isSectionExpanded));
		// TODO Ask UI folks for appropriate icons.  For time being I'm using two I
		// found in our icons library (one at 16x16 and the other at 11x11).
		this._expandStateImage.className = (this._isSectionExpanded ? "collapse-image" : "expand-image");
		// accessibility, provide alt and title text for images
		if (this._isSectionExpanded) {
			this._expandStateImage.setAttribute("title", this._messages["collapseTooltip"]);
			this._expandStateImage.setAttribute("alt", this._messages["collapseAlt"]);
		} else {
			this._expandStateImage.setAttribute("title", this._messages["expandTooltip"]);
			this._expandStateImage.setAttribute("alt", this._messages["expandAlt"]);
		}
	},

	// TODO Move the icons into CSS instead of hardcoded here in javascript.
	_getMaximizeStateImageSrc: function(isMaximized) {
		var imageSrc = null;
		if (isMaximized) {
			imageSrc = "com.ibm.rdm.web/sidebar/images/icons/dtool16/maximize.gif";
		} else {
			imageSrc = "com.ibm.rdm.web/sidebar/images/icons/etool16/maximize.gif";
		}
		return imageSrc;
	},

	_getExpandStateImageSrc: function(isExpanded) {
		var imageSrc = null;
		if (isExpanded) {
			imageSrc = "com.ibm.rdm.web/sidebar/images/twistie-open.gif";
		} else { // closed
			imageSrc = "com.ibm.rdm.web/sidebar/images/twistie-closed.gif";
		}
		return imageSrc;
	},

	_addSectionContent: function() {
		this._sectionContent.appendChild(this.contentNode);
		var widget = this._contentWidget;
		if (widget) {
			widget.startup();
		}
	},
	
	_toggleExpansionState: function() {
		this._isSectionExpanded = (!this._isSectionExpanded);
		if (!this._sectionContent.firstChild) {
			this._addSectionContent();
		}
		this._setExpandedStateIcon();
	},

	_onKeyPressSectionHeader: function(event) {
		if(event.charCode == dojo.keys.SPACE) { 
			dojo.stopEvent(event);
			this._onSectionHeaderClick(); 
		}
	},

	_onKeyPressMaximize: function(event) {
		if(event.charCode == dojo.keys.SPACE) { 
			dojo.stopEvent(event);
			this._showAsMaximized();
		}
	},

	_onKeyPressExpand: function(event) {
		if(event.charCode == dojo.keys.SPACE) { 
			dojo.stopEvent(event);
			this._onSectionHeaderClick(); 
		}
	},

	_onSectionHeaderClick: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		this._toggleExpansionState();
		if (this.isExpanded()) {
			this.setHeight(this.getCollapsedHeight());
		}
		this.sidebarLayout.redrawSectionTitleIcons(this);
		this.sidebarLayout.setAllExpandedSectionsToSameHeight(false, this);
		this._sectionContent.style.display = (this._isSectionExpanded ? "" : "none");
		this._adjustTitleArea();
	},
	
	_onSectionHeaderMouseOver: function(event) {
		dojo.addClass(this._sectionHeader, "section-header-hover");
	},

	_onSectionHeaderMouseOut: function(event) {
		dojo.removeClass(this._sectionHeader, "section-header-hover");
	}
});

})();
