/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.storyboard.StoryboardParser");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");


var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;


dojo.declare(
	"com.ibm.rdm.web.storyboard.StoryboardParser",
	null,
{
    // summary:
    //    This class is responsible for parsing the information found within
    //    a Storyboard artifact.
	// description:
	//    Given a doc that looks similar to this:
	//      <?xml version="1.0" encoding="ASCII"?>
	//      <story:Document
    //        ...
	//        <story:frame base:name="Enter CDClassics Web Site" key="_aIcpUeq5EdyQQ7iiiI2jTw"
	//            master="rrc71#_ruhcEeqhEdya3qF6cysudA">
	//          <base:description id="_tdLBoD7UEd20dK_SlC_9ng">
	//            <h:p id="_KxYikD7VEd20dK_SlC_9ng">Main page</h:p>
	//          </base:description>
	//          <PrimaryLayer/>
	//          <NoteLayer>
	//            <contents>
	//              <ui:Cursor key="_qFAP0Oq9EdyZKMvJAss25Q">
	//                <ui:Anchor percentX="92" percentY="58" ref="_aIcpUeq5EdyQQ7iiiI2jTw/_bd20gOqiEdya3qF6cysudA"/>
	//                <text>User clicks to start shopping</text>
	//              </ui:Cursor>
	//            </contents>
	//          </NoteLayer>
	//        </story:frame>
	//        <story:frame base:name="Second" key="_9OwM8cMLEd2mZvGFwhQktQ"
	//            master="https://9.29.196.119:9443/jazz/resources/rdm/resources/rrc615#_4QgwYcJSEd2obsvPfEXX7w">
	//          <PrimaryLayer/>
	//        </story:frame>
	//        ...
	//      </story:Document>
	//    This class parses this document into an array of property lists of the
	//    following format:
	//	    [
	//        {
	//          frameIndex: 0,
    //          frameName: "Enter CDClassics Web Site",
	//          key: "_aIcpUeq5EdyQQ7iiiI2jTw",
	//          pathlessMasterResourceURI: "rrc71",
	//          externalMasterResourceURI: null
	//        },
	//        {
	//          frameIndex: 1,
    //          frameName: "Second",
	//          key: "_9OwM8cMLEd2mZvGFwhQktQ",
	//          pathlessMasterResourceURI: null,
	//          externalMasterResourceURI: "https://9.29.196.119:9443/jazz/resources/rdm/resources/rrc615"
	//        },
	//        ...
	//      ]
	//    We refer to these property lists as "frameInfos".

	STORY_NAMESPACE: "http://jazz.net/xmlns/alm/rm/Storyboard/v0.1",
	STORY_QUALIFIER: "story",

    ///////////////////////////////////////////////////////////////////////////
    // API

    constructor: function(keywordArgs) {
		// keywordArgs.doc: A document representing the content of a storyboard
        this._frameInfos = this._parseDocument(keywordArgs.doc);
    },

	getFrameInfos: function() {
		// summary:
		//    Get information about each of the frames in the StoryBoard
		//    artifact.
		// description:
		//    See this class's documentation for what information is
		//    conveyed in each "frameInfo" element found in the returned
		//    array of frame infos.
		return this._frameInfos;
	},

    ///////////////////////////////////////////////////////////////////////////
    // Helpers

	_parseDocument: function(doc) {
		var self = this;
		var getFrameInfo = function(frameIndex, frameNode) {
			var isExternalResource = function(masterAttributeValue) {
				value = masterAttributeValue.toLowerCase();
				return (
					0 == value.indexOf("http:")
					|| 0 == value.indexOf("https:")
				);
			};
			var getPathlessMasterResourceURI = function(masterAttributeValue) {
				var pathlessMasterResourceURI = null;
				if (masterAttributeValue) {
					var index = masterAttributeValue.indexOf("#");
					if (index > 0) {
						pathlessMasterResourceURI = masterAttributeValue.substring(0, index);
					}
				}
				return pathlessMasterResourceURI;
			};
			var name = domutils.getAttributeNS(
				frameNode,
				"name",
				com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE,
				com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER);
			var description = self._getFrameDescription(frameNode);
			var key = frameNode.getAttribute("key");
			var master = frameNode.getAttribute("master");
			var pathlessMasterResourceURI = null;
			var externalMasterResourceURI = null;
			if (master) {
				if (isExternalResource(master)) {
					externalMasterResourceURI = master;
				} else {
					pathlessMasterResourceURI = getPathlessMasterResourceURI(master);
				}
			}
			return {
				frameIndex: frameIndex,
				frameName: (name || ""),
				frameDescription: (description || ""),
				key: key,
				pathlessMasterResourceURI: pathlessMasterResourceURI,
				externalMasterResourceURI: externalMasterResourceURI
			};
		}
		var frameNodes = domutils.getElementsByTagNameNS(
			doc, "frame",
			this.STORY_NAMESPACE, this.STORY_QUALIFIER);
		var frameNodeInfos = [];
		for (var i = 0; i < frameNodes.length; i++) {
			frameNodeInfos.push(getFrameInfo(i, frameNodes[i]));
		}
		return frameNodeInfos;
	},

	_getFrameDescription: function(frameNode) {
		// summary:
		//    Get a DOM frame node's description.
		// description:
		//    Here's an example of the content from which we would want to
		//    pull out "this is the first frame":
		//    <story:frame base:name="frame1" key="_nW0IkR-KEd6dMZa0BLb9WA">
		//      <base:description id="_iHq6cB-bEd6dMZa0BLb9WA">
		//        <h:p id="_iHq6cR-bEd6dMZa0BLb9WA">this is the first frame</h:p>
		//      </base:description>
		//      <PrimaryLayer>
		//        <contents>
		//          <ui:Radio key="_ghCT0B-bEd6dMZa0BLb9WA">
		//            <dgm:Point x="170" y="206"/>
		//            <size height="-1" width="-1"/>
		//          </ui:Radio>
		//        </contents>
		//      </PrimaryLayer>
		//    </story:frame>
		// returns:
		//    The description string associated with frameNode, or if not present, "".
		var getDescriptionString = function(descriptionNode) {
			var pNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
				descriptionNode, "p", queryutils.H_NAMESPACE, queryutils.H_QUALIFIER);
			return (
				(pNodes.length > 0 && pNodes[0].firstChild) ?
				pNodes[0].firstChild.nodeValue :
				null
			);
		};
		var getDescriptionIfParentNodeNameMatches = function(descriptionNode, parentNodeName) {
			return (
				(descriptionNode.parentNode.nodeName === parentNodeName) ?
				getDescriptionString(descriptionNode) :
				null
			);
		};
		var descriptionNodes =
			com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
				frameNode, "description", queryutils.BASE_NAMESPACE, queryutils.BASE_QUALIFIER);
		if (descriptionNodes.length > 0
			&& descriptionNodes[0].parentNode.nodeName === "story:frame"
			) {
			return getDescriptionString(descriptionNodes[0]);
		}
		return "";
	}
});

})();
